/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)cdr_stream.h    generated by: makeheader    Sat Mar  9 09:37:57 2024
 *
 *		built from:	stream.ifc
 */

#ifndef cdr_stream_h
#define cdr_stream_h


 #ifndef cdr_dll_h
    #include "cdr/cdr_dll.h"
 #endif
 #ifndef osapi_type_h
    #include "osapi/osapi_type.h"
 #endif
 #ifndef reda_buffer_h
    #include "reda/reda_buffer.h"
 #endif
 #ifndef reda_seqenceNumber_h
    #include "reda/reda_sequenceNumber.h"
 #endif
 #ifndef cdr_type_h
    #include "cdr/cdr_type.h"
 #endif 
 #ifndef cdr_encapsulation_h
    #include "cdr/cdr_encapsulation.h"
 #endif
 #ifndef reda_fastBuffer_h
    #include "reda/reda_fastBuffer.h"
 #endif
  #include "xcdr/xcdr_stream.h"

/* For VxWorks defines */
#ifdef RTI_VXWORKS
	#include <vxWorks.h>
#endif

#ifdef __cplusplus
    extern "C" {
#endif

struct REDABuffer;

struct RTICdrStreamState;

struct RTICdrStream; /* forward declare to appease the compiler */



/*e \ingroup RTICdrStreamModule
  Endian.
*/
typedef RTI_UINT8 RTICdrEndian;

/*e \ingroup RTICdrStreamModule
  Big endian in CDR.
*/
#define RTI_CDR_ENDIAN_BIG (0)

/*e \ingroup RTICdrStreamModule
  Little endian in CDR
*/
#define RTI_CDR_ENDIAN_LITTLE (1)

/*e \ingroup RTICdrStreamModule
  Default endian in CDR
*/
#ifdef RTI_ENDIAN_LITTLE
  #define RTI_CDR_ENDIAN_DEFAULT RTI_CDR_ENDIAN_LITTLE
#else
  #define RTI_CDR_ENDIAN_DEFAULT RTI_CDR_ENDIAN_BIG
#endif

/*e \ingroup RTICdrStreamModule
  Used with RTICdrStream_set()
*/
#define RTI_CDR_STREAM_ALIGNMENT (8)


extern RTICdrDllExport
void RTICdrStream_init(struct RTICdrStream *me);

extern RTICdrDllExport
struct RTICdrStream * RTICdrStream_new(void);

extern RTICdrDllExport
void RTICdrStream_delete(struct RTICdrStream *me);

extern RTICdrDllExport
void RTICdrStream_set(struct RTICdrStream *me, char *buffer,
		      unsigned int bufferLength);

extern RTICdrDllExport
void RTICdrStream_setContext(
        struct RTICdrStream *me,
        const void *context);

extern RTICdrDllExport
void RTICdrStream_copy(
        struct RTICdrStream *dest,
        const struct RTICdrStream *src);

extern RTICdrDllExport
void RTICdrStream_setEndian(struct RTICdrStream *me, RTICdrEndian endian);

extern RTICdrDllExport
RTICdrEndian RTICdrStream_getEndian(struct RTICdrStream *me);

extern RTICdrDllExport
void RTICdrStream_resetPosition(struct RTICdrStream *me);

extern RTICdrDllExport
void RTICdrStream_clearAndReset(struct RTICdrStream *me);

extern RTICdrDllExport
void RTICdrStream_print(const struct RTICdrStream *me);

extern RTICdrDllExport
RTIBool RTICdrStream_setCurrentPositionOffset(struct RTICdrStream *me,
					      int offset);

extern RTICdrDllExport
RTIBool RTICdrStream_incrementCurrentPosition(struct RTICdrStream *me,
					      int amount);

extern RTICdrDllExport
int RTICdrStream_getRemainder(const struct RTICdrStream *me);

extern RTICdrDllExport
int RTICdrStream_getCurrentPositionOffset(const struct RTICdrStream *me);

extern RTICdrDllExport
int RTICdrStream_getRelativeCurrentPositionOffset(const struct RTICdrStream *me);

extern RTICdrDllExport
char* RTICdrStream_getBuffer(const struct RTICdrStream *me);

extern RTICdrDllExport
int RTICdrStream_getBufferLength(const struct RTICdrStream *me);

extern RTICdrDllExport
int RTICdrStream_setBufferLength(struct RTICdrStream *me, int newLength);

extern RTICdrDllExport
void* RTICdrStream_getUserData(struct RTICdrStream *me);

extern RTICdrDllExport
void RTICdrStream_setUserData(struct RTICdrStream *me, void *newUserData);

extern RTICdrDllExport
RTIBool RTICdrStream_append(struct RTICdrStream *me,
			    const struct RTICdrStream *copyFrom,
			    int bytesToCopy);

extern RTICdrDllExport
RTIBool RTICdrStream_appendFull(struct RTICdrStream *me,
				const struct RTICdrStream *copy);

extern RTICdrDllExport
RTIBool RTICdrStream_checkSize(const struct RTICdrStream *me,
			       unsigned int size);

extern RTICdrDllExport
void RTICdrStream_popState(struct RTICdrStream *me,
                           const struct RTICdrStreamState *state);

extern RTICdrDllExport
void RTICdrStream_popStateNoAlign(struct RTICdrStream *me,
                                  const struct RTICdrStreamState *state);

extern RTICdrDllExport
void RTICdrStream_pushState(struct RTICdrStream *me,
                            struct RTICdrStreamState * state,
			    int newLength);

extern RTICdrDllExport
void RTICdrStreamState_init(struct RTICdrStreamState * state);

extern RTICdrDllExport
char* RTICdrStream_getCurrentPosition(const struct RTICdrStream *me);

extern RTICdrDllExport
char* RTICdrStream_setCurrentPosition(struct RTICdrStream *me, 
                                      char *newPosition);

extern RTICdrDllExport
char * RTICdrStream_resetAlignment(struct RTICdrStream *me);

extern RTICdrDllExport
char * RTICdrStream_resetAlignmentWithOffset(struct RTICdrStream *me, int offset);

extern RTICdrDllExport
void RTICdrStream_restoreAlignment(struct RTICdrStream *me,
                                   char * position);

extern RTICdrDllExport
RTIBool RTICdrStream_isDirty(struct RTICdrStream *me);

extern RTICdrDllExport
void RTICdrStream_setDirtyBit(
    struct RTICdrStream *me, RTIBool value);

extern RTICdrDllExport
RTIBool RTICdrStream_isZeroOnAlign(struct RTICdrStream *me);

extern RTICdrDllExport
void RTICdrStream_setZeroOnAlign(
    struct RTICdrStream *me, RTIBool value);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeChar(struct RTICdrStream *me,
				   const RTICdrChar *in);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeChar(struct RTICdrStream *me, 
                                     RTICdrChar *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeCharFromStream(struct RTICdrStream *me,
                                             struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalCharFromStream(const struct RTICdrStream *me,
                                         const struct RTICdrStream *other);

extern RTICdrDllExport
RTIBool RTICdrStream_skipChar(struct RTICdrStream *me);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeWchar(struct RTICdrStream *me, 
				    const RTICdrWchar *in);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeWchar(struct RTICdrStream *me,
				      RTICdrWchar *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeWcharFromStream(struct RTICdrStream *me,
                                              struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalWcharFromStream(const struct RTICdrStream *me,
                                          const struct RTICdrStream *other);

extern RTICdrDllExport
RTIBool RTICdrStream_skipWchar(struct RTICdrStream *me);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeCORBAWChar(struct RTICdrStream *me,
                                         const void *in,
					 RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeCORBAWChar(struct RTICdrStream *me,
				           void *out,
					   RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeOctet(struct RTICdrStream *me,
				    const RTICdrOctet *in);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeOctet(struct RTICdrStream *me,
				      RTICdrOctet *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeOctetFromStream(struct RTICdrStream *me,
                                              struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalOctetFromStream(const struct RTICdrStream *me,
                                          const struct RTICdrStream *other);

extern RTICdrDllExport
RTIBool RTICdrStream_skipOctet(struct RTICdrStream *me);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeShort(struct RTICdrStream *me,
				    const RTICdrShort *in );

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeShort(struct RTICdrStream *me,
				      RTICdrShort *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeShortFromStream(struct RTICdrStream *me,
                                              struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalShortFromStream(const struct RTICdrStream *me,
                                          const struct RTICdrStream *other);

extern RTICdrDllExport
RTIBool RTICdrStream_skipShort(struct RTICdrStream *me);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeUnsignedShort(struct RTICdrStream *me,
					    const RTICdrUnsignedShort *in );

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeUnsignedShort(struct RTICdrStream *me,
					      RTICdrUnsignedShort *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeUnsignedShortFromStream(struct RTICdrStream *me,
                                                      struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalUnsignedShortFromStream(const struct RTICdrStream *me,
                                                  const struct RTICdrStream *other);

extern RTICdrDllExport
RTIBool RTICdrStream_skipUnsignedShort(struct RTICdrStream *me);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeLong(struct RTICdrStream *me,
				   const RTICdrLong *in);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeLong(struct RTICdrStream *me, 
                                     RTICdrLong *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeLongFromStream(struct RTICdrStream *me,
                                             struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalLongFromStream(const struct RTICdrStream *me,
                                         const struct RTICdrStream *other);

extern RTICdrDllExport
RTIBool RTICdrStream_skipLong(struct RTICdrStream *me);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeUnsignedLong(struct RTICdrStream *me,
					   const RTICdrUnsignedLong *in);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeUnsignedLong(struct RTICdrStream *me,
					     RTICdrUnsignedLong *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeUnsignedLongFromStream(struct RTICdrStream *me,
                                                     struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalUnsignedLongFromStream(const struct RTICdrStream *me,
                                                 const struct RTICdrStream *other);

extern RTICdrDllExport
RTIBool RTICdrStream_skipUnsignedLong(struct RTICdrStream *me);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeLongLong(struct RTICdrStream *me,
				       const RTICdrLongLong *in );

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeLongLong(struct RTICdrStream *me,
					 RTICdrLongLong *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeLongLongFromStream(struct RTICdrStream *me,
                                                 struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalLongLongFromStream(const struct RTICdrStream *me,
                                             const struct RTICdrStream *other);

extern RTICdrDllExport
RTIBool RTICdrStream_skipLongLong(struct RTICdrStream *me);

extern RTICdrDllExport RTIBool
RTICdrStream_serializeUnsignedLongLong(struct RTICdrStream *me,
				       const RTICdrUnsignedLongLong *in );

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeUnsignedLongLong(struct RTICdrStream *me,
						 RTICdrUnsignedLongLong *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeUnsignedLongLongFromStream(struct RTICdrStream *me,
                                                         struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalUnsignedLongLongFromStream(const struct RTICdrStream *me,
                                                     const struct RTICdrStream *other);

extern RTICdrDllExport
RTIBool RTICdrStream_skipUnsignedLongLong(struct RTICdrStream *me);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeFloat(struct RTICdrStream *me,
				    const RTICdrFloat *in);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeFloat(struct RTICdrStream *me,
				      RTICdrFloat *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeFloatFromStream(struct RTICdrStream *me,
                                              struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalFloatFromStream(const struct RTICdrStream *me,
                                          const struct RTICdrStream *other);

extern RTICdrDllExport
RTIBool RTICdrStream_skipFloat(struct RTICdrStream *me);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeDouble(struct RTICdrStream *me,
				     const RTICdrDouble *in);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeDouble(struct RTICdrStream *me,
				       RTICdrDouble *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeDoubleFromStream(struct RTICdrStream *me,
                                               struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalDoubleFromStream(const struct RTICdrStream *me,
                                           const struct RTICdrStream *other);

extern RTICdrDllExport
RTIBool RTICdrStream_skipDouble(struct RTICdrStream *me);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeLongDouble(struct RTICdrStream *me,
					 const RTICdrLongDouble *in);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeLongDouble(struct RTICdrStream *me,
					   RTICdrLongDouble *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeLongDoubleFromStream(struct RTICdrStream *me,
                                                   struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalLongDoubleFromStream(const struct RTICdrStream *me,
                                               const struct RTICdrStream *other);

extern RTICdrDllExport
RTIBool RTICdrStream_skipLongDouble(struct RTICdrStream *me);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeBoolean(struct RTICdrStream *me, 
				      const RTICdrBoolean *in);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeBoolean(struct RTICdrStream *me, 
					RTICdrBoolean *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeBooleanFromStream(struct RTICdrStream *me,
                                               struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalBooleanFromStream(const struct RTICdrStream *me,
                                            const struct RTICdrStream *other);

extern RTICdrDllExport
RTIBool RTICdrStream_skipBoolean(struct RTICdrStream *me);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeEnum(struct RTICdrStream *me,
				   const RTICdrEnum *in);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeEnum(struct RTICdrStream *me,
				     RTICdrEnum *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeEnumFromStream(struct RTICdrStream *me,
                                             struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalEnumFromStream(const struct RTICdrStream *me,
                                         const struct RTICdrStream *other);

extern RTICdrDllExport
RTIBool RTICdrStream_skipEnum(struct RTICdrStream *me);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeVariableSizedBoolean(struct RTICdrStream *me,
		 				    const void *in,
						    RTICdrUnsignedLong size);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeVariableSizedBoolean(struct RTICdrStream *me,
					             void *out,
						     RTICdrUnsignedLong size);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeSequenceNumber(
      struct RTICdrStream *me,
      const struct REDASequenceNumber *sn);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeSequenceNumber(
      struct RTICdrStream *me,
      struct REDASequenceNumber *sn);

extern RTICdrDllExport
RTIBool RTICdrStream_serializePrimitiveArray(
    struct RTICdrStream *me,
    const void *in, RTICdrUnsignedLong length,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_serializePrimitiveArrayFast(
    struct RTICdrStream *me,
    const void *in, RTICdrUnsignedLong length,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializePrimitiveArray(
    struct RTICdrStream *me,
    void *out, RTICdrUnsignedLong length,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializePrimitiveArrayFast(
    struct RTICdrStream *me,
    void *out, RTICdrUnsignedLong length,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_serializePrimitiveArrayFromStream(
    struct RTICdrStream *me, struct RTICdrStream *in,
    RTICdrUnsignedLong length,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_equalPrimitiveArrayFromStream(
    struct RTICdrStream *me, struct RTICdrStream *other,
    RTICdrUnsignedLong length,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_equalZeroPrimitiveArray(
    struct RTICdrStream *me,
    RTICdrUnsignedLong length,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_skipPrimitiveArray(
    struct RTICdrStream *me,
    RTICdrUnsignedLong length,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_skipPrimitiveArrayAndGetLength(
    struct RTICdrStream *me,
    RTICdrUnsignedLong length,
    RTICdrPrimitiveType type,
    RTICdrUnsignedLong *result);

extern RTICdrDllExport
RTIBool RTICdrStream_serializePrimitivePointerArray(
    struct RTICdrStream *me,
    const void **in,
    RTICdrUnsignedLong length,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializePrimitivePointerArray(
    struct RTICdrStream *me,
    void ** out, RTICdrUnsignedLong length,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_skipPrimitivePointerArray(
    struct RTICdrStream *me,
    RTICdrUnsignedLong length,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeCORBAWCharArray(
    struct RTICdrStream *me,
    const void *in, RTICdrUnsignedLong length,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeCORBAWCharArray(
    struct RTICdrStream *me,
    void *out, RTICdrUnsignedLong length,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeVariableSizedBooleanArray(struct RTICdrStream *me,
                                                        const void *in,
                                                        RTICdrUnsignedLong length,
                                                        RTICdrUnsignedLong size);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeVariableSizedBooleanArray(struct RTICdrStream *me,
                                                          void *out,
                                                          RTICdrUnsignedLong length,
                                                          RTICdrUnsignedLong size);

extern RTICdrDllExport
RTIBool RTICdrStream_serializePrimitiveSequence(
    struct RTICdrStream *me,
    const void *in,
    RTICdrUnsignedLong numberOfElements,
    RTICdrUnsignedLong maximumLength,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_serializePrimitiveSequenceFast(
    struct RTICdrStream *me,
    const void *in,
    RTICdrUnsignedLong numberOfElements,
    RTICdrUnsignedLong maximumLength,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializePrimitiveSequence(
    struct RTICdrStream *me,
    void *out,
    RTICdrUnsignedLong *numberOfElements,
    RTICdrUnsignedLong maximumLength,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializePrimitiveSequenceFast(
    struct RTICdrStream *me,
    void *out,
    RTICdrUnsignedLong *numberOfElements,
    RTICdrUnsignedLong maximumLength,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_serializePrimitiveSequenceFromStream(
    struct RTICdrStream *me, struct RTICdrStream *in,
    RTICdrUnsignedLong *numberOfElements,
    RTICdrUnsignedLong maximumLength,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_equalPrimitiveSequenceFromStream(
    struct RTICdrStream *me, struct RTICdrStream *other,
    RTICdrUnsignedLong *numberOfElements,
    RTICdrUnsignedLong maximumLength,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_skipPrimitiveSequence(
    struct RTICdrStream *me,
    RTICdrUnsignedLong *numberOfElements,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_skipPrimitiveSequenceAndGetLength(
    struct RTICdrStream *me,
    RTICdrUnsignedLong *numberOfElements,
    RTICdrPrimitiveType type,
    RTICdrUnsignedLong *result);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeCORBAWCharSequence(
    struct RTICdrStream *me,
    const void *in,
    RTICdrUnsignedLong numberOfElements,
    RTICdrUnsignedLong maximumLength,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeCORBAWCharSequence(
    struct RTICdrStream *me,
    void *out,
    RTICdrUnsignedLong *numberOfElements,
    RTICdrUnsignedLong maximumLength,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_serializePrimitivePointerSequence(
    struct RTICdrStream *me,
    const void **in,
    RTICdrUnsignedLong numberOfElements,
    RTICdrUnsignedLong maximumLength,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializePrimitivePointerSequence(
    struct RTICdrStream *me,
    void **out,
    RTICdrUnsignedLong *numberOfElements,
    RTICdrUnsignedLong maximumLength,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_skipPrimitivePointerSequence(
    struct RTICdrStream *me,
    RTICdrUnsignedLong *numberOfElements,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeVariableSizedBooleanSequence(
    struct RTICdrStream *me,
    const void *in,
    RTICdrUnsignedLong numberOfElements,
    RTICdrUnsignedLong maximumLength,
    RTICdrUnsignedLong size);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeVariableSizedBooleanSequence(
    struct RTICdrStream *me,
    void *out,
    RTICdrUnsignedLong *numberOfElements,
    RTICdrUnsignedLong maximumLength,
    RTICdrUnsignedLong size);


typedef RTIBool (*RTICdrStreamSerializeFunction)(
        void *endpointData, 
        const void *in,
        struct RTICdrStream *stream,
        RTIBool serializeEncapsulation,
        RTIEncapsulationId encapsulationId,
        RTIBool serializeData, 
        void *endpointPluginQos);


typedef RTIBool (*RTICdrStreamDeserializeFunction)(
                void *endpointData,
                void *out,
                struct RTICdrStream *stream,
                RTIBool deserializeEncapsulation, 
                RTIBool deserializeSample,
                void *endpointPluginQos);


typedef RTIBool (*RTICdrStreamSkipFunction)(
                void *endpointData,
                struct RTICdrStream *me,
                RTIBool skipEncapsulation, 
                RTIBool skipSample,
                void * endpointPluginQos);

typedef RTIBool (*RTICdrStreamGetBufferWithSizeFunction)(
        void *userData,
        struct REDABuffer *buffer,
        RTICdrUnsignedLong bufferSize,
        struct REDAWorker *worker);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeNonPrimitiveArray(
    struct RTICdrStream *me, const void *in, RTICdrUnsignedLong length,
    unsigned int elementSize, RTICdrStreamSerializeFunction serializeFunction,
    RTIBool serializeEncapsulation, 
    RTIEncapsulationId encapsulationId,
    RTIBool serializeSample, 
    void *endpointData, void *endpointPluginQos);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeNonPrimitiveArray(
    struct RTICdrStream *me, void *out, RTICdrUnsignedLong length,
    unsigned int elementSize,
    RTICdrStreamDeserializeFunction deserializeFunction,
    RTIBool deserializeEncapsulation, RTIBool deserializeSample,
    void *endpointData, void *endpointPluginQos);

extern RTICdrDllExport
RTIBool RTICdrStream_skipNonPrimitiveArray(
    struct RTICdrStream *me,
    RTICdrUnsignedLong length,
    RTICdrStreamSkipFunction skipFunction,
    RTIBool skipEncapsulation, RTIBool skipSample,
    void *endpointData, void *endpointPluginQos);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeNonPrimitivePointerArray(
    struct RTICdrStream *me, const void ** in, RTICdrUnsignedLong length,
    unsigned int elementSize, RTICdrStreamSerializeFunction serializeFunction,
    RTIBool serializeEncapsulation, 
    RTIEncapsulationId encapsulationId,
    RTIBool serializeData, 
    void *endpointData, void *endpointPluginQos);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeNonPrimitivePointerArray(
    struct RTICdrStream *me, void **out, RTICdrUnsignedLong length,
    unsigned int elementSize,
    RTICdrStreamDeserializeFunction deserializeFunction,
    RTIBool deserializeEncapsulation, RTIBool deserializeSample,
    void *endpointData, void *endpointPluginQos);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeNonPrimitiveSequence(
    struct RTICdrStream *me, const void *in,
    RTICdrUnsignedLong numberOfElements, RTICdrUnsignedLong maximumLength,
    unsigned int elementSize,
    RTICdrStreamSerializeFunction serializeFunction,
    RTIBool serializeEncapsulation, 
    RTIEncapsulationId encapsulationId,    
    RTIBool serializeData, 
    void *endpointData, void *endpointPluginQos);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeNonPrimitiveSequence(
    struct RTICdrStream *me, void *out, RTICdrUnsignedLong *numberOfElements,
    RTICdrUnsignedLong maximumLength, unsigned int elementSize,
    RTICdrStreamDeserializeFunction deserializeFunction,
    RTIBool deserializeEncapsulation, RTIBool deserializeSample,
    void *endpointData, void *endpointPluginQos);

extern RTICdrDllExport
RTIBool RTICdrStream_skipNonPrimitiveSequence(
    struct RTICdrStream *me, 
    RTICdrUnsignedLong *numberOfElements,
    RTICdrStreamSkipFunction skipFunction,
    RTIBool skipEncapsulation, RTIBool skipSample,
    void *endpointData, void *endpointPluginQos);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeNonPrimitivePointerSequence(
    struct RTICdrStream *me, const void **in,
    RTICdrUnsignedLong numberOfElements, RTICdrUnsignedLong maximumLength,
    RTICdrStreamSerializeFunction serializeFunction,
    RTIBool serializeEncapsulation, 
    RTIEncapsulationId encapsulationId,
    RTIBool serializeData,
    void *endpointData, void *endpointPluginQos);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeNonPrimitivePointerSequence(
    struct RTICdrStream *me, void **out, RTICdrUnsignedLong *numberOfElements,
    RTICdrUnsignedLong maximumLength,
    RTICdrStreamDeserializeFunction deserializeFunction,
    RTIBool deserializeEncapsulation, RTIBool deserializeSample,
    void *endpointData, void *endpointPluginQos);

extern RTICdrDllExport
RTIBool RTICdrStream_skipNonPrimitivePointerSequence(
    struct RTICdrStream *me, 
    RTICdrUnsignedLong *numberOfElements,
    RTICdrStreamSkipFunction skipFunction,
    RTIBool deserializeEncapsulation, RTIBool deserializeSample,
    void *endpointData, void *endpointPluginQos);

 
typedef RTIBool (*RTICdrTypeGetDeserializedSampleSizeFunction)(
    void * endpointData,
    unsigned int * size,
    RTIBool skipEncapsulation,
    RTIBool skipSample,
    unsigned int currentAlignment,
    RTIBool onlyMembers,
    struct RTICdrStream *stream,
    void *endpointPluginQos);

extern RTICdrDllExport RTIBool
RTICdrStream_getNonPrimitiveSequenceDeserializedSize(
    void * endpointData,
    unsigned int * size,
    RTICdrTypeGetDeserializedSampleSizeFunction
        getDeserializedSampleSizeFunction,
    unsigned int elementSize,
    unsigned int currentAlignment,
    struct RTICdrStream *stream,
    void *endpointPluginQos);


typedef RTIBool (*RTICdrTypeInitializeDeserializationBufferPointersFromStream)(
    void * endpoint_data,
    void * sample,
    RTIBool skip_encapsulation,
    RTIBool skip_sample,
    struct REDABufferManager *buffer_manager,
    struct RTICdrStream *stream,
    void *endpoint_plugin_qos);

extern RTICdrDllExport RTIBool
RTICdrStream_initializeDesBufferPointersFromStreamInNonPrimitiveSeq(
    void * endpointData,
    void **bufferPtr,
    RTICdrUnsignedLong * sequenceLength,
    RTICdrTypeInitializeDeserializationBufferPointersFromStream
        initDesBufferFunction,
    unsigned int elementSize,
    struct REDABufferManager *buffer_manager,
    struct RTICdrStream *stream,
    void *endpointPluginQos);


typedef RTIBool (*RTICdrTypeInitializeDeserializationBufferPointersFromSample)(
    void * endpoint_data,
    void * dstSample,
    struct REDABufferManager *buffer_manager,
    void * srcSample);

extern RTICdrDllExport RTIBool
RTICdrStream_initializeDesBufferPointersFromSampleInNonPrimitiveSeq(
    void * endpointData,
    void **dstBufferPtr,
    const void * srcBuffer,
    RTICdrUnsignedLong sequenceLength,
    RTICdrTypeInitializeDeserializationBufferPointersFromSample
        initDesBufferFunction,
    unsigned int elementSize,
    struct REDABufferManager *buffer_manager);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeString(struct RTICdrStream *me,
				     const RTICdrChar *in,
				     RTICdrUnsignedLong maximumLength);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeString(struct RTICdrStream *me,
				       RTICdrChar *out,
				       RTICdrUnsignedLong maximumLength);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeStringEx(
    struct RTICdrStream *me, 
    RTICdrChar **out, 
    RTICdrUnsignedLong maximumLength,
    RTIBool realloc);


/*e \ingroup RTICdrStreamSafeSerializeFromStreamModule
  Re-serialize a string type from one stream to another.
*/
extern RTICdrDllExport
RTIBool RTICdrStream_serializeStringFromStream(struct RTICdrStream *me,
                                               struct RTICdrStream *in,
                                               RTICdrUnsignedLong maximumLength);

extern RTICdrDllExport
RTIBool RTICdrStream_equalStringFromStream(struct RTICdrStream *me,
                                           struct RTICdrStream *other,
                                           RTICdrUnsignedLong maximumLength);

extern RTICdrDllExport
RTIBool RTICdrStream_skipString(struct RTICdrStream *me,
                                       RTICdrUnsignedLong maximumLength);

extern RTICdrDllExport
RTIBool RTICdrStream_skipStringAndGetLength(struct RTICdrStream *me,
    RTICdrUnsignedLong maximumLength, RTICdrUnsignedLong *result);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeWstring(struct RTICdrStream *me,
				      const RTICdrWchar *in,
				      RTICdrUnsignedLong maximumLength);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeWstring(struct RTICdrStream *me,
					RTICdrWchar *out,
					RTICdrUnsignedLong maximumLength);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeWstringEx(
    struct RTICdrStream *me, 
    RTICdrWchar **out, 
    RTICdrUnsignedLong maximumLength,
    RTIBool realloc);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeWstringFromStream(struct RTICdrStream *me,
                                                struct RTICdrStream *in,
                                                RTICdrUnsignedLong maximumLength);

extern RTICdrDllExport
RTIBool RTICdrStream_equalWstringFromStream(struct RTICdrStream *me,
                                            struct RTICdrStream *other,
                                            RTICdrUnsignedLong maximumLength);

extern RTICdrDllExport
RTIBool RTICdrStream_equalZeroWstring(struct RTICdrStream *me);

extern RTICdrDllExport
RTIBool RTICdrStream_skipWstring(struct RTICdrStream *me,
                                 RTICdrUnsignedLong maximumLength);

extern RTICdrDllExport
RTIBool RTICdrStream_skipWstringAndGetLength(struct RTICdrStream *me,
                                 RTICdrUnsignedLong *result);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeStringArray(struct RTICdrStream *me,
                                          const void* in, 
                                          RTICdrUnsignedLong length,
                                          RTICdrUnsignedLong maximumStringLength,
                                          RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeStringArray(struct RTICdrStream *me,
                                            void *out,
                                            RTICdrUnsignedLong length,
                                            RTICdrUnsignedLong maximumStringLength,
                                            RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeStringArrayEx(struct RTICdrStream *me,
                                            void *out,
                                            RTICdrUnsignedLong length,
                                            RTICdrUnsignedLong maximumStringLength,
                                            RTICdrPrimitiveType type,
                                            RTIBool reallocate);

extern RTICdrDllExport
RTIBool RTICdrStream_skipStringArray(struct RTICdrStream *me,
                                     RTICdrUnsignedLong length,
                                     RTICdrUnsignedLong maximumStringLength,
                                     RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_skipStringArrayAndGetLength(
                                     struct RTICdrStream *me,
                                     RTICdrUnsignedLong length,
                                     RTICdrUnsignedLong maximumStringLength,
                                     RTICdrPrimitiveType type,
                                     RTICdrUnsignedLong *result);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeStringPointerArray(struct RTICdrStream *me,
                                                 const void** in, 
                                                 RTICdrUnsignedLong length,
                                                 RTICdrUnsignedLong maximumStringLength,
                                                 RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeStringPointerArray(struct RTICdrStream *me,
                                                   void **out,
                                                   RTICdrUnsignedLong length,
                                                   RTICdrUnsignedLong maximumStringLength,
                                                   RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeStringPointerArrayEx(struct RTICdrStream *me,
                                                   void **out,
                                                   RTICdrUnsignedLong length,
                                                   RTICdrUnsignedLong maximumStringLength,
                                                   RTICdrPrimitiveType type,
                                                   RTIBool reallocate);

extern RTICdrDllExport
RTIBool RTICdrStream_skipStringPointerArray(struct RTICdrStream *me,
                                            RTICdrUnsignedLong length,
                                            RTICdrUnsignedLong maximumStringLength,
                                            RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeStringSequence(
    struct RTICdrStream *me,
    const void *in,
    RTICdrUnsignedLong numberOfElements,
    RTICdrUnsignedLong maximumLength,
    RTICdrUnsignedLong maximumStringLength,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeStringSequence(
    struct RTICdrStream *me,
    void *out,
    RTICdrUnsignedLong *numberOfElements,
    RTICdrUnsignedLong maximumLength,
    RTICdrUnsignedLong maximumStringLength,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeStringSequenceEx(
    struct RTICdrStream *me,
    void *out,
    RTICdrUnsignedLong *numberOfElements,
    RTICdrUnsignedLong maximumLength,
    RTICdrUnsignedLong maximumStringLength,
    RTICdrPrimitiveType type,
    RTIBool reallocate);

extern RTICdrDllExport
RTIBool RTICdrStream_skipStringSequence(
    struct RTICdrStream *me,
    RTICdrUnsignedLong *numberOfElements,
    RTICdrUnsignedLong maximumStringLength,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_skipStringSequenceAndGetLength(
    struct RTICdrStream *me,
    RTICdrUnsignedLong *numberOfElements,
    RTICdrUnsignedLong maximumStringLength,
    RTICdrPrimitiveType type,
    RTICdrUnsignedLong *result);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeStringPointerSequence(
    struct RTICdrStream *me,
    const void **in,
    RTICdrUnsignedLong numberOfElements,
    RTICdrUnsignedLong maximumLength,
    RTICdrUnsignedLong maximumStringLength,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeStringPointerSequence(
    struct RTICdrStream *me,
    void **out,
    RTICdrUnsignedLong *numberOfElements,
    RTICdrUnsignedLong maximumLength,
    RTICdrUnsignedLong maximumStringLength,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeStringPointerSequenceEx(
    struct RTICdrStream *me,
    void **out,
    RTICdrUnsignedLong *numberOfElements,
    RTICdrUnsignedLong maximumLength,
    RTICdrUnsignedLong maximumStringLength,
    RTICdrPrimitiveType type,
    RTIBool reallocate);

extern RTICdrDllExport
RTIBool RTICdrStream_skipStringPointerSequence(
    struct RTICdrStream *me,
    RTICdrUnsignedLong *numberOfElements,
    RTICdrUnsignedLong maximumStringLength,
    RTICdrPrimitiveType type);

extern RTICdrDllExport
void RTICdrStream_serializeCharFast(struct RTICdrStream *me,
				    const RTICdrChar *in);

extern RTICdrDllExport
void RTICdrStream_deserializeCharFast(struct RTICdrStream *me, RTICdrChar *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeCharFromStreamFast(struct RTICdrStream *me,
                                                 struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalCharFromStreamFast(const struct RTICdrStream *me,
                                             const struct RTICdrStream *other);

extern RTICdrDllExport
void RTICdrStream_skipCharFast(struct RTICdrStream *me);

extern RTICdrDllExport
void RTICdrStream_serializeWcharFast(struct RTICdrStream *me,
				     const RTICdrWchar *in);

extern RTICdrDllExport
void RTICdrStream_deserializeWcharFast(struct RTICdrStream *me,
				       RTICdrWchar *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeWcharFromStreamFast(struct RTICdrStream *me,
                                                  struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalWcharFromStreamFast(const struct RTICdrStream *me,
                                              const struct RTICdrStream *other);

extern RTICdrDllExport
void RTICdrStream_skipWcharFast(struct RTICdrStream *me);

extern RTICdrDllExport
void RTICdrStream_serializeOctetFast(struct RTICdrStream *me,
				     const RTICdrOctet *in);

extern RTICdrDllExport
void RTICdrStream_deserializeOctetFast(struct RTICdrStream *me,
				       RTICdrOctet *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeOctetFromStreamFast(struct RTICdrStream *me,
                                                  struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalOctetFromStreamFast(const struct RTICdrStream *me,
                                              const struct RTICdrStream *other);

extern RTICdrDllExport
void RTICdrStream_skipOctetFast(struct RTICdrStream *me);

extern RTICdrDllExport
void RTICdrStream_serializeShortFast(struct RTICdrStream *me,
				     const RTICdrShort *in);

extern RTICdrDllExport
void RTICdrStream_deserializeShortFast(struct RTICdrStream *me,
				       RTICdrShort *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeShortFromStreamFast(struct RTICdrStream *me,
                                                  struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalShortFromStreamFast(const struct RTICdrStream *me,
                                              const struct RTICdrStream *other);

extern RTICdrDllExport
void RTICdrStream_skipShortFast(struct RTICdrStream *me);

extern RTICdrDllExport
void RTICdrStream_serializeUnsignedShortFast(struct RTICdrStream *me,
					     const RTICdrUnsignedShort *in);

extern RTICdrDllExport
void RTICdrStream_deserializeUnsignedShortFast(struct RTICdrStream *me,
					       RTICdrUnsignedShort *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeUnsignedShortFromStreamFast(struct RTICdrStream *me,
                                                          struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalUnsignedShortFromStreamFast(const struct RTICdrStream *me,
                                                      const struct RTICdrStream *other);

extern RTICdrDllExport
void RTICdrStream_skipUnsignedShortFast(struct RTICdrStream *me);

extern RTICdrDllExport
void RTICdrStream_serializeLongFast(struct RTICdrStream *me,
				    const RTICdrLong *in);

extern RTICdrDllExport
void RTICdrStream_deserializeLongFast(struct RTICdrStream *me, 
                                      RTICdrLong *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeLongFromStreamFast(struct RTICdrStream *me,
                                                 struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalLongFromStreamFast(const struct RTICdrStream *me,
                                             const struct RTICdrStream *other);

extern RTICdrDllExport
void RTICdrStream_skipLongFast(struct RTICdrStream *me);

extern RTICdrDllExport
void RTICdrStream_serializeUnsignedLongFast(struct RTICdrStream *me,
					    const RTICdrUnsignedLong *in);

extern RTICdrDllExport
void RTICdrStream_deserializeUnsignedLongFast(struct RTICdrStream *me,
					      RTICdrUnsignedLong *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeUnsignedLongFromStreamFast(struct RTICdrStream *me,
                                                         struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalUnsignedLongFromStreamFast(const struct RTICdrStream *me,
                                                     const struct RTICdrStream *other);

extern RTICdrDllExport
void RTICdrStream_skipUnsignedLongFast(struct RTICdrStream *me);

extern RTICdrDllExport
void RTICdrStream_serializeLongLongFast(struct RTICdrStream *me,
					const RTICdrLongLong *in);

extern RTICdrDllExport
void RTICdrStream_deserializeLongLongFast(struct RTICdrStream *me,
					  RTICdrLongLong *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeLongLongFromStreamFast(struct RTICdrStream *me,
                                                     struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalLongLongFromStreamFast(const struct RTICdrStream *me,
                                                 const struct RTICdrStream *other);

extern RTICdrDllExport
void RTICdrStream_skipLongLongFast(struct RTICdrStream *me);

extern RTICdrDllExport
void RTICdrStream_serializeUnsignedLongLongFast(
    struct RTICdrStream *me, const RTICdrUnsignedLongLong *in);

extern RTICdrDllExport
void RTICdrStream_deserializeUnsignedLongLongFast(struct RTICdrStream *me,
						  RTICdrUnsignedLongLong *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeUnsignedLongLongFromStreamFast(struct RTICdrStream *me,
                                                             struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalUnsignedLongLongFromStreamFast(const struct RTICdrStream *me,
                                                         const struct RTICdrStream *other);

extern RTICdrDllExport
void RTICdrStream_skipUnsignedLongLongFast(struct RTICdrStream *me);

extern RTICdrDllExport
void RTICdrStream_serializeFloatFast(struct RTICdrStream *me,
				     const RTICdrFloat *in);

extern RTICdrDllExport
void RTICdrStream_deserializeFloatFast(struct RTICdrStream *me,
				       RTICdrFloat *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeFloatFromStreamFast(struct RTICdrStream *me,
                                                  struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalFloatFromStreamFast(const struct RTICdrStream *me,
                                              const struct RTICdrStream *other);

extern RTICdrDllExport
void RTICdrStream_skipFloatFast(struct RTICdrStream *me);

extern RTICdrDllExport
void RTICdrStream_serializeDoubleFast(struct RTICdrStream *me,
				      const RTICdrDouble *in);

extern RTICdrDllExport
void RTICdrStream_deserializeDoubleFast(struct RTICdrStream *me,
					RTICdrDouble *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeDoubleFromStreamFast(struct RTICdrStream *me,
                                                   struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalDoubleFromStreamFast(const struct RTICdrStream *me,
                                               const struct RTICdrStream *other);

extern RTICdrDllExport
void RTICdrStream_skipDoubleFast(struct RTICdrStream *me);

extern RTICdrDllExport
void RTICdrStream_serializeLongDoubleFast(struct RTICdrStream *me,
					  const RTICdrLongDouble *in);

extern RTICdrDllExport
void RTICdrStream_deserializeLongDoubleFast(struct RTICdrStream *me,
					    RTICdrLongDouble *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeLongDoubleFromStreamFast(struct RTICdrStream *me,
                                                       struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalLongDoubleFromStreamFast(const struct RTICdrStream *me,
                                                   const struct RTICdrStream *other);

extern RTICdrDllExport
void RTICdrStream_skipLongDoubleFast(struct RTICdrStream *me);

extern RTICdrDllExport
void RTICdrStream_serializeBooleanFast(struct RTICdrStream *me,
				       const RTICdrBoolean *in);

extern RTICdrDllExport
void RTICdrStream_deserializeBooleanFast(struct RTICdrStream *me,
					 RTICdrBoolean *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeBooleanFromStreamFast(struct RTICdrStream *me,
                                                    struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalBooleanFromStreamFast(const struct RTICdrStream *me,
                                                const struct RTICdrStream *other);

extern RTICdrDllExport
void RTICdrStream_skipBooleanFast(struct RTICdrStream *me);

extern RTICdrDllExport
void RTICdrStream_serializeEnumFast(struct RTICdrStream *me,
				    const RTICdrEnum *in);

extern RTICdrDllExport
void RTICdrStream_deserializeEnumFast(struct RTICdrStream *me,
				      RTICdrEnum *out);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeEnumFromStreamFast(struct RTICdrStream *me,
                                                 struct RTICdrStream *in);

extern RTICdrDllExport
RTIBool RTICdrStream_equalEnumFromStreamFast(const struct RTICdrStream *me,
                                             const struct RTICdrStream *other);

extern RTICdrDllExport
void RTICdrStream_skipEnumFast(struct RTICdrStream *me);

extern RTICdrDllExport
RTIBool RTICdrStream_align(struct RTICdrStream *me, int alignment);

extern RTICdrDllExport
RTIBool RTICdrStream_isAligned(
        struct RTIXCdrStream *me,
        RTIOsapiAlignment alignment);

extern RTICdrDllExport
RTIBool RTICdrStream_align_ex(
            struct RTICdrStream *me,
            int alignment,
            RTIBool zeroOnAlign);

extern RTICdrDllExport
void RTICdrStream_alignFast(struct RTICdrStream *me, int alignment);

extern RTICdrDllExport
void RTICdrStream_serializeUnsignedShortToBigEndianFast(struct RTICdrStream *me, 
                                                        const RTICdrUnsignedShort *in);

extern RTICdrDllExport
void RTICdrStream_deserializeUnsignedShortFromBigEndianFast(struct RTICdrStream *me, 
                                                            RTICdrUnsignedShort *out);

extern RTICdrDllExport
void RTICdrStream_serializeUnsignedLongToBigEndianFast(struct RTICdrStream *me, 
                                                       const RTICdrUnsignedLong *in);

extern RTICdrDllExport
void RTICdrStream_deserializeUnsignedLongFromBigEndianFast(struct RTICdrStream *me, 
                                                           RTICdrUnsignedLong *out);

extern RTICdrDllExport
void RTICdrStream_serializeBigEndianToUnsignedLongFast(struct RTICdrStream *me, 
                                                       const RTICdrOctet *in);

extern RTICdrDllExport
void RTICdrStream_deserializeBigEndianFromUnsignedLongFast(struct RTICdrStream *me, 
                                                           RTICdrOctet *out);

extern RTICdrDllExport 
RTIBool RTICdrStream_serializeUnsignedShortToBigEndian(struct RTICdrStream *me, 
                                                       const RTICdrUnsignedShort *in);

extern RTICdrDllExport 
RTIBool RTICdrStream_deserializeUnsignedShortFromBigEndian(struct RTICdrStream *me, 
                                                           RTICdrUnsignedShort *out);

extern RTICdrDllExport 
RTIBool RTICdrStream_serializeUnsignedLongToBigEndian(struct RTICdrStream *me, 
                                                      const RTICdrUnsignedLong *in);

extern RTICdrDllExport 
RTIBool RTICdrStream_deserializeUnsignedLongFromBigEndian(struct RTICdrStream *me, 
                                                          RTICdrUnsignedLong *out);

extern RTICdrDllExport 
RTIBool RTICdrStream_setEndianFromEncapsulationDefault(
    struct RTICdrStream *me);

extern RTICdrDllExport 
RTIBool RTICdrStream_serializeCdrEncapsulationDefault(
    struct RTICdrStream *me);

extern RTICdrDllExport 
RTIBool RTICdrStream_serializeEncapsulation(
    struct RTICdrStream *me, unsigned short *kind, unsigned short *options);

extern RTICdrDllExport 
RTIBool RTICdrStream_serializeCdrEncapsulation(
    struct RTICdrStream *me, unsigned short *kind, unsigned short *options);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeAndSetCdrEncapsulation(
    struct RTICdrStream *me, RTIEncapsulationId encapsulationId);

extern RTICdrDllExport 
RTIBool RTICdrStream_deserializeCdrEncapsulationAndSetDefault(
    struct RTICdrStream *me);

extern RTICdrDllExport 
RTIBool RTICdrStream_deserializeAndSetCdrEncapsulation(
    struct RTICdrStream *me);

extern RTICdrDllExport
RTIBool RTICdrStream_skipEncapsulation(
    struct RTICdrStream *me);

extern RTICdrDllExport 
RTIBool RTICdrStream_deserializeEncapsulation(
    struct RTICdrStream *me, unsigned short *kind, unsigned short *options);

extern RTICdrDllExport 
RTIBool RTICdrStream_deserializeCdrEncapsulation(
    struct RTICdrStream *me, unsigned short *kind, unsigned short *options);

extern RTICdrDllExport 
RTIBool RTICdrStream_deserializeParameterCdrEncapsulationAndSetDefault(
    struct RTICdrStream *me);

extern RTICdrDllExport
void RTICdrStream_pushEndian(
    struct RTICdrStream *me, RTICdrEndian *endian, RTIBool *needByteSwap);

extern RTICdrDllExport
void RTICdrStream_popEndian(
    struct RTICdrStream *me, RTICdrEndian *endian, RTIBool *needByteSwap);

extern RTICdrDllExport
RTIBool RTICdrStream_setCdrEncapsulationKind(
    struct RTICdrStream *me, unsigned short kind, unsigned short options);

extern RTICdrDllExport
RTIBool RTICdrStream_setEncapsulationKind(
    struct RTICdrStream *me, unsigned short kind, unsigned short options);

extern RTICdrDllExport
unsigned short RTICdrStream_getEncapsulationKind(
    struct RTICdrStream *me);

extern RTICdrDllExport
unsigned short RTICdrStream_getEncapsulationOptions(
    struct RTICdrStream *me);

extern RTICdrDllExport
void RTICdrStream_getEncapsulationSize(unsigned int size);

extern RTICdrDllExport
void RTICdrStream_computeMD5(
    struct RTICdrStream *me, RTICdrOctet * out);

extern RTICdrDllExport
RTIBool RTICdrStream_deserializeParameterHeader(
    struct RTICdrStream *me,
    struct RTICdrStreamState * state,
    RTICdrUnsignedLong * parameterId,
    RTICdrUnsignedLong * length,
    RTIBool * extended,
    RTIBool * mustUnderstand);

extern RTICdrDllExport
void RTICdrStream_moveToNextParameterHeader(
    struct RTICdrStream *me,
    const struct RTICdrStreamState * state,
    RTICdrUnsignedLong length);

extern RTICdrDllExport
char * RTICdrStream_serializeParameterHeader(
    struct RTICdrStream *me,
    struct RTICdrStreamState * state,
    RTIBool extended,
    RTICdrUnsignedLong parameterId,
    RTIBool mustUnderstand);

extern RTICdrDllExport
RTIBool RTICdrStream_serializeParameterLength(
    struct RTICdrStream *me,
    const struct RTICdrStreamState * state,
    RTIBool extended,
    char * lengthPosition);

extern RTICdrDllExport
unsigned int RTICdrStream_getExtendedParameterHeaderMaxSizeSerialized(
    unsigned int currentAlignment);

extern RTICdrDllExport
unsigned int RTICdrStream_getParameterHeaderMaxSizeSerialized(
    unsigned int currentAlignment);

extern RTICdrDllExport
RTIBool RTICdrStream_lookUnsignedLong(
    struct RTICdrStream *me,
    RTICdrUnsignedLong * value);

extern RTICdrDllExport
RTIBool RTICdrStream_peekEncapsulationId(
        struct RTICdrStream *me,
        RTIXCdrEncapsulationId *id);

extern RTICdrDllExport
RTIBool RTICdrStream_isCompressionEnabledOnEncapsulation(struct RTICdrStream *me);

extern RTICdrDllExport
void RTICdrStream_setEncapsulationOptionsCompressionId(
        struct RTICdrStream *me,
        RTIOsapiCompressionPluginClassId compressionId);

extern RTICdrDllExport
RTIOsapiCompressionPluginClassId
RTICdrStream_getEncapsulationOptionsCompressionId(
        const struct RTICdrStream *me);

extern RTICdrDllExport
void RTICdrEncapsulationOptions_pack(
        RTIEncapsulationOptions *options,
        RTICdrOctet serializedDataNumPaddingBytes,
        RTIOsapiCompressionPluginClassId compressionId,
        RTICdrOctet compressionNumPaddingBytes);

extern RTICdrDllExport
void RTICdrEncapsulationOptions_unpack(
        RTIEncapsulationOptions options,
        RTICdrOctet *serializedDataNumPaddingBytes,
        RTIOsapiCompressionPluginClassId *compressionId,
        RTICdrOctet *compressionNumPaddingBytes);

extern RTICdrDllExport
void RTICdrStream_packEncapsulationOptions(
        struct RTICdrStream *me,
        RTICdrOctet serializedDataNumPaddingBytes,
        RTIOsapiCompressionPluginClassId compressionId,
        RTICdrOctet compressionNumPaddingBytes);

extern RTICdrDllExport
void RTICdrStream_unpackEncapsulationOptions(
        const struct RTICdrStream *me,
        RTICdrOctet *serializedDataNumPaddingBytes,
        RTIOsapiCompressionPluginClassId *compressionId,
        RTICdrOctet *compressionNumPaddingBytes);

extern RTICdrDllExport RTIBool RTICdrStream_uncompress(
        struct RTICdrStream *me,
        RTICdrStreamGetBufferWithSizeFunction getBufferWithSize,
        void *userData,
        struct REDABuffer *uncompressBuffer,
        RTI_UINT32 maxUncompressedSize,
        struct REDAWorker *worker);

extern RTICdrDllExport RTIBool RTICdrStream_goToPid(
        struct RTICdrStream *me,
        RTICdrUnsignedShort pidToFind,
        RTICdrUnsignedShort pidSentinel,
        RTIBool moveToBody);


#ifdef __cplusplus
    }	/* extern "C" */
#endif

  #include "cdr/cdr_stream_impl.h"

#endif /* cdr_stream_h */
