/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)dds_c_monitoring_impl.h    generated by: makeheader    Sat Mar  9 09:39:09 2024
 *
 *		built from:	monitoring_impl.ifc
 */

#ifndef dds_c_monitoring_impl_h
#define dds_c_monitoring_impl_h


  #ifndef dds_c_infrastructure_h
    #include "dds_c/dds_c_infrastructure.h"
  #endif
  #ifndef ndds_config_c_h
    #include "ndds/ndds_config_c.h"
  #endif
  #ifndef dds_c_domain_h
    #include "dds_c/dds_c_domain.h"
  #endif
  #ifndef dds_c_infrastructure_h
    #include "dds_c/dds_c_infrastructure.h"
  #endif
#ifdef __cplusplus
    extern "C" {
#endif


#define RTI_MONITORING_RESOURCE_FULLY_QUALIFIED_NAME_MAX_SIZE 1024
#define RTI_MONITORING_RESOURCE_NAME_MAX_LENGTH 255


typedef struct RTI_MonitoringClass * (*RTI_Monitoring_CreateFunction)(
        DDS_DomainParticipantFactory *factory,
        const struct RTI_MonitoringProperty_t *property);

typedef DDS_Boolean (*RTI_Monitoring_PrefinalizeFunction)(
        struct RTI_MonitoringClass *self);


typedef void (*RTI_Monitoring_DeleteFunction)(
        struct RTI_MonitoringClass *self);


typedef DDS_Boolean (*RTI_Monitoring_IsSharedLibraryFunction)(void);

typedef struct RTI_MonitoringResource *(
        *RTI_Monitoring_AssertResourceFunction)(
        struct RTI_MonitoringClass *monitoringClass,
        DDS_UnsignedLong *concurrencyIndex,
        const struct DDS_GUID_t *userResourceGuid,
        DDS_UnsignedLong classId,
        const char *resourceName,
        void *userResource,
        struct RTI_MonitoringResource *parentResource,
        const struct RTI_MonitoringResourceForwarder *forwarder);


typedef void (*RTI_Monitoring_DeleteResourceFunction)(
        struct RTI_MonitoringClass *monitoringClass,
        struct RTI_MonitoringResource *resource);


typedef void (*RTI_Monitoring_DeleteResourceChildrenFunction)(
        struct RTI_MonitoringClass *monitoringClass,
        struct RTI_MonitoringResource *resource);

typedef DDS_Boolean (*RTI_Monitoring_PushEventMetricFunction)(
        struct RTI_MonitoringClass *monitoringClass,
        void *applicationMetricGroup,
        struct RTI_MonitoringResource *resource,
        DDS_UnsignedLong metricGroupIndex);

typedef DDS_Boolean (*RTI_Monitoring_onLoggingCollectionChangeFunction)(
        struct RTI_MonitoringClass *monitoringClass);

typedef const struct RTI_MonitoringProperty_t *(
        *RTI_Monitoring_GetPropertyFunction)(
        const struct RTI_MonitoringClass *monitoringClass);


typedef DDS_Boolean (*RTI_Monitoring_DispatchCommandFunction)(
        struct RTI_MonitoringClass *monitoringClass,
        const struct DDS_ServiceRequest *sample,
        const struct DDS_SampleInfo *info);

typedef DDS_Boolean (*RTI_Monitoring_PauseResourceMetricsFunction)(
        struct RTI_MonitoringClass *monitoringClass,
        struct RTI_MonitoringResource *resource);


typedef DDS_Boolean (*RTI_Monitoring_PauseResourceChildrenMetricsFunction)(
        struct RTI_MonitoringClass *monitoringClass,
        struct RTI_MonitoringResource *resource);


typedef DDS_Boolean (*RTI_Monitoring_ResumeResourceMetricsFunction)(
        struct RTI_MonitoringClass *monitoringClass,
        struct RTI_MonitoringResource *resource);


typedef DDS_Boolean (*RTI_Monitoring_PostLogMessage)(
        struct RTI_MonitoringClass *monitoringClass,
        const char *text,
        NDDS_Config_SyslogLevel severity,
        DDS_UnsignedLong message_id,
        const struct DDS_Duration_t *timestamp,
        NDDS_Config_LogFacility facility,
        const void *structured_data,
        const char *category,
        DDS_UnsignedLong concurrencyIndex);

struct RTI_MonitoringClass {
    DDS_DomainParticipantFactory *factory;
    struct RTI_MonitoringResource *rootResource;
    RTI_Monitoring_PrefinalizeFunction prefinalizeFunction;
    RTI_Monitoring_DeleteFunction deleteFunction;
    RTI_Monitoring_IsSharedLibraryFunction isSharedLibraryFunction;
    RTI_Monitoring_AssertResourceFunction assertResourceFunction;
    RTI_Monitoring_DeleteResourceFunction deleteResourceFunction;
    RTI_Monitoring_DeleteResourceChildrenFunction
            deleteResourceChildrenFunction;
    RTI_Monitoring_PushEventMetricFunction pushEventMetricFunction;
    RTI_Monitoring_onLoggingCollectionChangeFunction onLoggingCollectionChange;
    RTI_Monitoring_GetPropertyFunction getPropertyFunction;
    RTI_Monitoring_DispatchCommandFunction dispatchCommandFunction;
    RTI_Monitoring_PauseResourceMetricsFunction pauseResourceMetricsFunction;
    RTI_Monitoring_PauseResourceChildrenMetricsFunction
            pauseResourceChildrenMetricsFunction;
    RTI_Monitoring_ResumeResourceMetricsFunction resumeResourceMetricsFunction;
    RTI_Monitoring_PostLogMessage postLogMessage;
};


struct RTI_MonitoringGlobalsConcurrency {
    /*
     * - This array of mutexes will be created in the function
     *   RTI_Monitoring_enable_with_property() with the size provided by the
     *   user in MonitoringProperty_t_concurrency_level.
     * - Once this array is created, it will be kept alive during the whole life
     *   of the application, independently if RTI_Monitoring_disable is
     *   called.
     * - This array will be removed in DDS_DomainParticipantFactory_deleteI,
     *   if the Monitoring Library has been disabled properly.
     * - If the user enables the Monitoring Library again with a different value
     *   for MonitoringProperty_t_concurrency_level.
     *     - A warning log message will be printed explaining the new level will
     *       not be used.
     *     - The current array will continue be used.
     */
    struct RTIOsapiSemaphore **mutexPool;
    DDS_UnsignedLong level;
};


struct RTI_MonitoringGlobals {
    RTI_Monitoring_CreateFunction createFunction;
    void *libraryHandle;
    struct RTI_MonitoringClass *monitoringClass;
    struct RTIOsapiSemaphore *mutex;
    struct RTI_MonitoringGlobalsConcurrency eventConcurrency;
    struct RTI_MonitoringGlobalsConcurrency loggingConcurrency;
    /*
     * A counter that will be increased each time the Library is enabled. It is
     * used to know if a MonitoringResourceHandle is still valid, since it
     * could belong to a previous Monitoring Library activation.
     */
    DDS_UnsignedLongLong libraryEpoch;
    /* This clock will be Monotonic if available. System otherwise */
    struct RTIClock *eventClock;

    RTIOsapiThreadTssFactory *tssFactory;
    /*
     * Thread specific storage key used to indicate if we are within the enable
     * operation. This is necessary to avoid deadlocks when combining explicit
     * calls to the RTI_Monitoring_enable operation with configuration using
     * XML and the attribute is_default_participant_factory_profile.
     */
    RTI_UINT32 inEnableTssKey;
};


#define RTI_MonitoringGlobalsConcurrency_INITIALIZER { \
    NULL, /* mutexPool */ \
    0 /* level */ \
}

#define RTI_MonitoringGlobals_INITIALIZER \
  { \
    NULL,                                         /* createFunction */ \
    NULL,                                         /* libraryHandle */ \
    NULL,                                         /* monitoringClass */ \
    NULL,                                         /* mutex */ \
    RTI_MonitoringGlobalsConcurrency_INITIALIZER, /* eventConcurrency */ \
    RTI_MonitoringGlobalsConcurrency_INITIALIZER, /* loggingConcurrency */ \
    0,                                            /* libraryEpoch */ \
    NULL,                                         /* eventClock */ \
    NULL,                                         /* tssFactory */ \
    0                                             /* inEnableTssKey */ \
  }

extern DDSCDllExport
struct RTI_MonitoringGlobals * RTI_MonitoringGlobals_get_instance(void);

extern DDSCDllExport RTI_UINT32
RTI_MonitoringGlobalsConcurrency_getConcurrencyIndex(
        struct RTI_MonitoringGlobalsConcurrency *self,
        RTI_UINT32 number);

extern DDSCDllExport
RTIBool RTI_MonitoringGlobalsConcurrency_initialize(
        struct RTI_MonitoringGlobalsConcurrency *self,
        RTI_UINT32 level);

extern DDSCDllExport
void RTI_MonitoringGlobalsConcurrency_finalize(
        struct RTI_MonitoringGlobalsConcurrency *self);


typedef enum RTI_MonitoringLockKind {
    RTI_MONITORING_LOCK_KIND_LIBRARY = 0,
    RTI_MONITORING_LOCK_KIND_METRIC,
    RTI_MONITORING_LOCK_KIND_LOGGING
} RTI_MonitoringLockKind;

extern DDSCDllExport
RTI_MonitoringLockResult RTI_Monitoring_lockI(
        RTI_MonitoringLockKind kind,
        DDS_UnsignedLong index,
        DDS_Boolean checkEnable,
        DDS_Boolean logDebugMessages,
        const struct DDS_Duration_t *timeout);

extern DDSCDllExport RTI_MonitoringLockResult RTI_Monitoring_unlockI(
        RTI_MonitoringLockKind kind,
        DDS_UnsignedLong index,
        DDS_Boolean checkEnable);

extern DDSCDllExport DDS_Boolean RTI_Monitoring_getDataReaderMetricGroup(
        void **applicationMetricGroup,
        const void *userResource,
        DDS_UnsignedLong metricGroupIndex);

extern DDSCDllExport DDS_Boolean RTI_Monitoring_returnDataReaderMetricGroup(
        void **applicationMetricGroup,
        DDS_UnsignedLong metricGroupIndex);

extern DDSCDllExport DDS_Boolean RTI_Monitoring_getDataWriterMetricGroup(
        void **applicationMetricGroup,
        const void *userResource,
        DDS_UnsignedLong metricGroupIndex);

extern DDSCDllExport DDS_Boolean RTI_Monitoring_returnDataWriterMetricGroup(
        void **applicationMetricGroup,
        DDS_UnsignedLong metricGroupIndex);

extern DDSCDllExport DDS_Boolean RTI_Monitoring_getDomainParticipantMetricGroup(
        void **applicationMetricGroup,
        const void *userResource,
        DDS_UnsignedLong metricGroupIndex);

extern DDSCDllExport DDS_Boolean RTI_Monitoring_getPublisherMetricGroup(
        void **applicationMetricGroup,
        const void *userResource,
        DDS_UnsignedLong metricGroupIndex);

extern DDSCDllExport DDS_Boolean RTI_Monitoring_getSubscriberMetricGroup(
        void **applicationMetricGroup,
        const void *userResource,
        DDS_UnsignedLong metricGroupIndex);

extern DDSCDllExport DDS_Boolean RTI_Monitoring_getTopicMetricGroup(
        void **applicationMetricGroup,
        const void *userResource,
        DDS_UnsignedLong metricGroupIndex);

extern DDSCDllExport DDS_Boolean RTI_Monitoring_returnTopicMetricGroup(
        void **applicationMetricGroup,
        DDS_UnsignedLong metricGroupIndex);

extern DDSCDllExport void RTI_Monitoring_pushEventMetricToDds(
        const void *presMetricGroup,
        void *presResource,
        RTI_UINT32 classId,
        RTI_UINT32 metricGroupIndex);

extern DDSCDllExport struct RTI_MonitoringResourceHandle
DDS_Entity_get_monitoring_resource_handle(const DDS_Entity *entity);

extern DDSCDllExport void RTI_Monitoring_notify_logging_collection_change(void);

extern DDSCDllExport void RTI_Monitoring_post_log_message(
        const char *text,
        NDDS_Config_SyslogLevel severity,
        DDS_UnsignedLong message_id,
        const struct DDS_Duration_t *timestamp,
        NDDS_Config_LogFacility facility,
        const void *structured_data,
        const char *category);


#define RTI_Monitoring_is_enabled() \
    (RTI_MonitoringGlobals_get_instance()->monitoringClass != NULL)

#define RTI_Monitoring_lock() \
    RTI_Monitoring_lockI( \
            RTI_MONITORING_LOCK_KIND_LIBRARY, \
            0, /* index */ \
            DDS_BOOLEAN_TRUE, /* checkEnable */ \
            DDS_BOOLEAN_TRUE, /* logDebugMessages */ \
            NULL /* timeout */)

#define RTI_Monitoring_unlock() \
    RTI_Monitoring_unlockI( \
            RTI_MONITORING_LOCK_KIND_LIBRARY, \
            0, /* index */ \
            DDS_BOOLEAN_TRUE /* checkEnable */)

/*ci
 * @brief Check that the Monitoring Library is enabled and lock the
 * Monitoring metric lock associated with the index.
 * @warning This lock is not reentrant, and calling this operation
 * more than once from the same threads may lead to deadlocks.
 * @param index \st_in. The metric queue index
 * @return \idref_MonitoringLockResult.
 * @see RTI_Monitoring_metric_unlock
 */
#define RTI_Monitoring_metric_lock(index__) \
    RTI_Monitoring_lockI( \
            RTI_MONITORING_LOCK_KIND_METRIC, \
            (index__), \
            DDS_BOOLEAN_TRUE, /* checkEnable */ \
            DDS_BOOLEAN_TRUE, /* logDebugMessages */ \
            NULL /* timeout */)

/*ci
 * @brief Check that the Monitoring Library is enabled and unlock the
 * Monitoring metric lock associated with the index.
 * There is one lock per metric queue.
 * @warning This lock is not reentrant, and calling this operation
 * more than once from the same threads may lead to deadlocks.
 * @param index \st_in. The metric queue index
 * @return \idref_MonitoringLockResult.
 * @see RTI_Monitoring_metric_lock
 */
#define RTI_Monitoring_metric_unlock(index__) \
    RTI_Monitoring_unlockI( \
            RTI_MONITORING_LOCK_KIND_METRIC, \
            (index__), \
            DDS_BOOLEAN_TRUE /* checkEnable */)

/*ci
 * @brief Check that the Monitoring Library is enabled and lock the
 * Monitoring logging lock associated with the index.
 * @warning This lock is not reentrant, and calling this operation
 * more than once from the same threads may lead to deadlocks.
 * @param index \st_in. The logging queue index
 * @return \idref_MonitoringLockResult.
 * @see RTI_Monitoring_logging_unlock
 */
#define RTI_Monitoring_logging_lock(index__) \
    RTI_Monitoring_lockI( \
            RTI_MONITORING_LOCK_KIND_LOGGING, \
            (index__), \
            DDS_BOOLEAN_TRUE, /* checkEnable */ \
            DDS_BOOLEAN_TRUE, /* logDebugMessages */ \
            NULL /* timeout */)

/*ci
 * @brief Check that the Monitoring Library is enabled and unlock the
 * Monitoring logging lock associated with the index.
 * There is one lock per logging queue.
 * @warning This lock is not reentrant, and calling this operation
 * more than once from the same threads may lead to deadlocks.
 * @param index \st_in. The logging queue index
 * @return \idref_MonitoringLockResult.
 * @see RTI_Monitoring_logging_lock
 */
#define RTI_Monitoring_logging_unlock(index__) \
    RTI_Monitoring_unlockI( \
            RTI_MONITORING_LOCK_KIND_LOGGING, \
            (index__), \
            DDS_BOOLEAN_TRUE /* checkEnable */)

/* 
 * Implementation note: The macro implementation checks for level == 0 
 * because otherwise if (self__)->level) is 0, the result of the macro
 * may lead to a numeric exception depending on the platform.
 *
 * Under normal circumstances, (self__)->level) should never be 0, but it is
 * set to zero in the tests that try to create an RTI_MonitoringResourceRegistry
 * directly without initializing the monitoring globals. For example, the
 * tests defined in MonitoringUtilsTester.c.
*/
#define RTI_MonitoringGlobalsConcurrency_getConcurrencyIndex(self__, number__) \
    (((self__)->level == 0) ? 0 : ((number__) % (self__)->level))

#define RTI_MonitoringResourceHandle_is_nil(self__) \
  ((self__).resource == RTI_MONITORING_RESOURCE_HANDLE_NIL.resource \
          && (self__).concurrencyIndex \
                  == RTI_MONITORING_RESOURCE_HANDLE_NIL.concurrencyIndex \
          && (self__).libraryEpoch \
                  == RTI_MONITORING_RESOURCE_HANDLE_NIL.libraryEpoch)

/*ci
 * @brief Check whether a Resource Handle is valid or not. A handle is
 * considered valid if it is not NIL and if its libraryEpoch is equal to the
 * libraryEpoch in the monitoringGlobals.
 */
#define RTI_MonitoringResourceHandle_is_valid(self__) \
  (!RTI_MonitoringResourceHandle_is_nil(self__) \
          && RTI_MonitoringGlobals_get_instance()->libraryEpoch \
                  == (self__).libraryEpoch)

#define RTI_MONITORING_STRING_METRIC_MAX_SIZE 255

/*
 * The root resource will be posted in the queue with index 0.
 * It will be the first task posted in the event task executor, so having index
 * 0 will ensure it will be the first one processed.
 * We use this number because it will always exist, as the concurrency_level
 * should be greater than 0.
 */
#define RTI_MONITORING_ROOT_RESOURCE_CONCURRENCY_INDEX 0


struct RTI_Monitoring_FloatStat {
    DDS_UnsignedLong period_ms;
    DDS_UnsignedLongLong count;
    DDS_Float mean;
    DDS_Float min;
    DDS_Float max;
};

struct RTI_Monitoring_ProcessMemoryUtilization {
    DDS_UnsignedLongLong resident_memory_bytes;
    DDS_UnsignedLongLong virtual_memory_bytes;
};

extern DDSCDllExport
struct RTI_MonitoringProperty_t * RTI_MonitoringProperty_t_copy_ex(
        struct RTI_MonitoringProperty_t *self,
        const struct RTI_MonitoringProperty_t *from,
        DDS_Boolean shallow);

extern DDSCDllExport
void RTI_MonitoringProperty_t_finalize_ex(
        struct RTI_MonitoringProperty_t *self,
        DDS_Boolean shallow);


struct DDS_QosProvider;

extern DDSCDllExport
RTIBool RTI_MonitoringProperty_fromMonitoringQosPolicy(
        struct RTI_MonitoringProperty_t *self,
        const struct DDS_MonitoringQosPolicy *policy,
        struct DDS_QosProvider *qosProvider);


#ifdef __cplusplus
    }	/* extern "C" */
#endif

#endif /* dds_c_monitoring_impl_h */
