/*

(c) Copyright, Real-Time Innovations, 2023.
All rights reserved.

No duplications, whole or partial, manual or electronic, may be made
without express written permission.  Any such copies, or
revisions thereof, must display this notice unaltered.
This code contains trade secrets of Real-Time Innovations, Inc.
*/

#ifndef RTI_DIST_LOGGER_IMPL_HPP_
#define RTI_DIST_LOGGER_IMPL_HPP_

#include <rti/config/Logger.hpp>
#include <rti/distlogger/DistLoggerOptions.hpp>

namespace rti { namespace dist_logger {

class RTIDLCPP2DllExport DistLoggerImpl {
public:
    DistLoggerImpl();

    static void close();

    static void set_options(const rti::dist_logger::DistLoggerOptions& options);

    void set_filter_level(int new_level);
    
    void set_print_format(const rti::config::PrintFormat& log_format);

    void set_verbosity(const rti::config::LogCategory& category, const rti::config::Verbosity& verbosity);

    void log_message(int logLevel, const char * message, const char * category);

    void log_message(int logLevel, const std::string& message, const std::string& category);

    void log_message(int logLevel, const char * message, const char * category, const dds::core::Time timestamp);

    void log_message(int logLevel, const std::string& message, const std::string& category, const dds::core::Time timestamp);
   
    void fatal(const char * message); 

    void fatal(const std::string& message);

    void severe(const char * message);

    void severe(const std::string& message);
    
    void error(const char * message);

    void error(const std::string& message);

    void warning(const char * message);

    void warning(const std::string& message);

    void notice(const char * message);

    void notice(const std::string& message);

    void info(const char * message);

    void info(const std::string& message);

    void debug(const char * message);

    void debug(const std::string& message);

    void trace(const char * message);

    void trace(const std::string& message);

    void log(int log_level, const char * message);

    void log(int log_level, const std::string& message);

    int get_fatal_log_level();

    int get_error_log_level();

    int get_warning_log_level();

    int get_notice_log_level();

    int get_info_log_level();

    int get_debug_log_level();

private:
    static void close_contained_distlogger_entities();

    static struct RTI_DL_DistLogger *native_;

    // We need to keep a reference to the Modern C++ DP
    // so that it's not destroyed after being passed to C
    static dds::domain::DomainParticipant participant_;
};
} } // namespace rti::dist_logger

#endif // RTI_DIST_LOGGER_IMPL_HPP_