/* $Id$

(c) Copyright, Real-Time Innovations, 2022.
All rights reserved.

No duplications, whole or partial, manual or electronic, may be made
without express written permission.  Any such copies, or
revisions thereof, must display this notice unaltered.
This code contains trade secrets of Real-Time Innovations, Inc.
*/

#ifndef RTI_DDS_TOPIC_TRUST_ENDPOINT_TRUST_ALGORITHM_INFO_HPP_
#define RTI_DDS_TOPIC_TRUST_ENDPOINT_TRUST_ALGORITHM_INFO_HPP_

// IMPORTANT: macros.hpp must be the first RTI header included in every header
// file so that symbols are exported correctly on Windows
#include <dds/core/macros.hpp>

#include "dds_c/dds_c_infrastructure_impl.h"

#include <dds/core/types.hpp>
#include <dds/core/Value.hpp>
#include <dds/core/Exception.hpp>
#include <rti/core/NativeValueType.hpp>
#include <rti/topic/trust/EndpointTrustInterceptorAlgorithmInfo.hpp>

namespace rti { namespace topic { namespace trust {

class EndpointTrustAlgorithmInfoAdapter {
public:
    typedef struct DDS_EndpointTrustAlgorithmInfo native_type;

    static void initialize(native_type& native_value)
    {
        static const native_type DEFAULT =
                DDS_EndpointTrustAlgorithmInfo_INITIALIZER_NOSEC;
        native_value = DEFAULT;
    }

    static void finalize(native_type&)
    {
        // nothing to do
    }

    static void copy(native_type& destination, const native_type& source)
    {
        DDS_EndpointTrustAlgorithmInfo_copy(&destination, &source);
    }

    static bool equals(const native_type& first, const native_type& second)
    {
        return DDS_EndpointTrustAlgorithmInfo_equals(&first, &second)
                == DDS_BOOLEAN_TRUE;
    }
};

class EndpointTrustAlgorithmInfo;

} } // namespace topic::trust

namespace core {

template <>
struct native_type_traits<rti::topic::trust::EndpointTrustAlgorithmInfo> {
    typedef rti::topic::trust::EndpointTrustAlgorithmInfoAdapter adapter_type;
    typedef DDS_EndpointTrustAlgorithmInfo native_type;
};

}

namespace topic { namespace trust {

/**
 * @ingroup DDSBuiltInTopicCommonModule
 * @RTI_class_definition class EndpointTrustAlgorithmInfo
 * @headerfile rti/topic/trust/EndpointTrustAlgorithmInfo.hpp
 *
 * @brief @extension @st_value_type Trust Plugins algorithm information
 * associated with the discovered endpoint.
 *
 * @details \dref_details_EndpointTrustAlgorithmInfo
 */
class OMG_DDS_API EndpointTrustAlgorithmInfo : public rti::core::NativeValueType<EndpointTrustAlgorithmInfo> {
public:
    RTI_NATIVE_VALUE_TYPE_DEFINE_DEFAULT_MOVE_OPERATIONS(EndpointTrustAlgorithmInfo)

    typedef rti::core::NativeValueType<EndpointTrustAlgorithmInfo> Base;
    /**
     * @brief Create an instance with the default Trust Algorithm Info
     * associated with the discovered endpoint.
     *
     * @details \dref_details_EndpointTrustAlgorithmInfo_new
     */
    EndpointTrustAlgorithmInfo() = default;

    EndpointTrustAlgorithmInfo(
            const struct DDS_EndpointTrustAlgorithmInfo& native_algo)
      : Base(native_algo)
    {
    }

    /**
     * @brief Get the Trust Plugins algorithms for interception of data and
     * metadata exchanged by the endpoint.
     *
     * @details \dref_details_EndpointTrustAlgorithmInfo_interceptor
     */
    EndpointTrustInterceptorAlgorithmInfo interceptor() const
    {
        return native().interceptor;
    }

};

} } }

#endif // RTI_DDS_TOPIC_TRUST_ENDPOINT_TRUST_ALGORITHM_INFO_HPP_
