/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)osapi_fileTracker.h    generated by: makeheader    Sat Mar  9 09:37:08 2024
 *
 *		built from:	fileTracker.ifc
 */

#ifndef osapi_fileTracker_h
#define osapi_fileTracker_h


#include "log/log_makeheader.h"

#include "osapi/osapi_dll.h"
#include "osapi/osapi_type.h"
#include "osapi/osapi_thread.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
    /* File Tracker has not observed any relevant event for the file. */
    RTI_OSAPI_FILE_TRACKER_EVENT_KIND_FILE_NONE,
    /*
     * File Tracker has detected the file has transitioned from existing
     * to not existing (i.e., the file has been deleted).
     */
    RTI_OSAPI_FILE_TRACKER_EVENT_KIND_FILE_DELETED,
    /*
     * File Tracker has detected the file has transitioned from not existing
     * to existing. This will happen either if an already tracked non-existing
     * file has been created, but also when the File Tracker first starts
     * tracking a file that already exists.
     */
    RTI_OSAPI_FILE_TRACKER_EVENT_KIND_FILE_NEW,
    /*
     * File Tracker has detected the file has changed in any way (except
     * being deleted or new, which is covered by other event kinds).
     */
    RTI_OSAPI_FILE_TRACKER_EVENT_KIND_FILE_CHANGED
} RTIOsapiFileTrackerEventKind;

struct RTIOsapiFileTrackerEvent {
    RTIOsapiFileTrackerEventKind kind;
    const char *fileName;
};


#define RTIOsapiFileTrackerEvent_INITIALIZER \
  { \
    RTI_OSAPI_FILE_TRACKER_EVENT_KIND_FILE_NONE, /* kind */ \
    NULL                                         /* fileName */ \
  }

typedef enum {
    /*
     * Default behavior, the tracker will rely on asynchronous OS notification
     * if supported, and polling if not.
     */
    RTI_OSAPI_FILE_TRACKER_DETECTION_KIND_AUTO,
    /* Polls tracked files periodically to track their status. */
    RTI_OSAPI_FILE_TRACKER_DETECTION_KIND_POLLING,
    /* Relies on asynchronous OS notifications to track files' status. */
    RTI_OSAPI_FILE_TRACKER_DETECTION_KIND_OS_NOTIFICATION
} RTIOsapiFileTrackerDetectionKind;

struct RTIOsapiFileTracker;

typedef void (*RTIOsapiFileTracker_OnFileChangeCallback)(
        void *listenerData,
        const struct RTIOsapiFileTrackerEvent *fileTrackerEvent);

struct RTIOsapiFileTrackerListener {
    void *listenerData;
    RTIOsapiFileTracker_OnFileChangeCallback onFileChange;
};


#define RTIOsapiFileTrackerListener_INITIALIZER \
  { \
    NULL, /* listenerData */ \
    NULL  /* onFileChange */ \
  }

struct RTIOsapiFileTrackerProperty {
    /*
     * Determines how the File Tracker will track files.
     */
    RTIOsapiFileTrackerDetectionKind detectionKind;
    /*
     * Poll period in milliseconds to check for file status changes. Only used
     * if detectionKind is either POLLING or AUTO.
     */
    RTI_UINT32 pollPeriodMillisec;
    /* Name of the thread managing File Tracker events. */
    char threadName[RTI_OSAPI_THREAD_NAME_MAX_SIZE];
};


#define RTIOsapiFileTrackerProperty_INITIALIZER \
  { \
    RTI_OSAPI_FILE_TRACKER_DETECTION_KIND_AUTO, /* detectionKind */ \
    500,                                        /* pollPeriodMillisec */ \
    ""                                          /* threadName */ \
  }

extern RTIOsapiDllExport struct RTIOsapiFileTracker *RTIOsapiFileTracker_new(
        const struct RTIOsapiFileTrackerProperty *property);

extern RTIOsapiDllExport void RTIOsapiFileTracker_delete(
        struct RTIOsapiFileTracker *self);

extern RTIOsapiDllExport RTIBool RTIOsapiFileTracker_startTrackingFile(
        struct RTIOsapiFileTracker *self,
        const struct RTIOsapiFileTrackerListener *listener,
        const char *fileName);

extern RTIOsapiDllExport RTIBool RTIOsapiFileTracker_stopTrackingFile(
        struct RTIOsapiFileTracker *self,
        const struct RTIOsapiFileTrackerListener *listener,
        const char *fileName);

extern RTIOsapiDllExport
RTIBool RTIOsapiFileTracker_stopTrackingAllFilesPerListener(
        struct RTIOsapiFileTracker *self,
        const struct RTIOsapiFileTrackerListener *listener);

extern RTIOsapiDllExport
RTIBool RTIOsapiFileTracker_stopTrackingAllFiles(
        struct RTIOsapiFileTracker *self);


#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* osapi_fileTracker_h */
