/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)osapi_hash.h    generated by: makeheader    Sat Mar  9 09:37:08 2024
 *
 *		built from:	hash.ifc
 */

#ifndef osapi_hash_h
#define osapi_hash_h


  #include <errno.h>
  #ifndef osapi_type_h
    #include "osapi/osapi_type.h"
  #endif
  #ifndef osapi_dll_h
    #include "osapi/osapi_dll.h"
  #endif

  #include <stdio.h>
  #include <stdarg.h>

#ifdef __cplusplus
    extern "C" {
#endif


#define RTI_OSAPI_HASH_MAX_LENGTH   (16)
#define RTI_OSAPI_HASH_VALUE_NIL    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}

#define RTI_OSAPI_HASH_FLAG_VALID	(0x01 << 0)



typedef struct RTIOsapiHash {
    RTI_UINT8 value[RTI_OSAPI_HASH_MAX_LENGTH];
    RTI_UINT8 length;
    RTI_UINT8 flags;
} RTIOsapiHash;


#define RTI_OSAPI_HASH_NIL \
    {                           \
    RTI_OSAPI_HASH_VALUE_NIL,   \
    0,                          \
    0                           \
    }

extern RTIOsapiDllExport
void RTIOsapiHash_initialize(RTIOsapiHash *self);

extern RTIOsapiDllExport
RTIBool RTIOsapiHash_equals(const RTIOsapiHash *a, const RTIOsapiHash *b);

extern RTIOsapiDllExport
int RTIOsapiHash_compare(const RTIOsapiHash *a, const RTIOsapiHash *b);

extern RTIOsapiDllExport
void RTIOsapiHash_copy(struct RTIOsapiHash *out, const struct RTIOsapiHash *in);

extern RTIOsapiDllExport
RTIBool RTIOsapiHash_isValid(const RTIOsapiHash *self);

extern RTIOsapiDllExport
RTI_UINT8 RTIOsapiHash_length(const RTIOsapiHash *self);

extern RTIOsapiDllExport
RTI_UINT8 *RTIOsapiHash_value(const RTIOsapiHash *self);

extern RTIOsapiDllExport
void RTIOsapiHash_reset(struct RTIOsapiHash *self);

extern RTIOsapiDllExport
RTIBool RTIOsapiHash_toString(
        const struct RTIOsapiHash *self,
        char *buffer,
        size_t *bufferSize);

extern RTIOsapiDllExport
void RTIOsapiHash_print(const struct RTIOsapiHash *self);

extern RTIOsapiDllExport void RTIOsapiHash_computeMd5(
        RTIOsapiHash *self,
        const char *in,
        size_t size);

extern RTIOsapiDllExport void RTIOsapiHash_computeScatterMd5(
        RTIOsapiHash *self,
        const char **in,
        size_t size,
        const size_t *elementSize);

typedef RTI_UINT8 RTIOsapiHashAlgorithmKind;


#define RTI_OSAPI_HASH_ALGORITHM_KIND_INVALID ((RTIOsapiHashAlgorithmKind) 0)


#define RTI_OSAPI_HASH_ALGORITHM_KIND_CRC32C ((RTIOsapiHashAlgorithmKind) 1)


#define RTI_OSAPI_HASH_ALGORITHM_KIND_XZ ((RTIOsapiHashAlgorithmKind) 2)


#define RTI_OSAPI_HASH_ALGORITHM_KIND_MD5 ((RTIOsapiHashAlgorithmKind) 3)

typedef RTI_UINT8 RTIOsapiHashStreamState;


#define RTI_OSAPI_HASH_STREAM_STATE_INVALID ((RTIOsapiHashStreamState) 0)


#define RTI_OSAPI_HASH_STREAM_STATE_INITIALIZED ((RTIOsapiHashStreamState) 1)


#define RTI_OSAPI_HASH_STREAM_STATE_STARTED ((RTIOsapiHashStreamState) 2)


#define RTI_OSAPI_HASH_STREAM_STATE_FINISHED ((RTIOsapiHashStreamState) 3)

typedef struct RTIOsapiHashStream {
    void *_internalState;
    RTIOsapiHashAlgorithmKind kind;
    RTIOsapiHashStreamState state;
    RTIOsapiHash hash;
} RTIOsapiHashStream;


#define RTIOsapiHashStream_INITIALIZER \
  { \
    NULL,                                  /* _internalState */ \
    RTI_OSAPI_HASH_ALGORITHM_KIND_INVALID, /* kind */ \
    RTI_OSAPI_HASH_STREAM_STATE_INVALID,   /* state */ \
    RTI_OSAPI_HASH_NIL                     /* hash */ \
  }

extern RTIOsapiDllExport RTIBool RTIOsapiHashStream_initialize(
        RTIOsapiHashStream *self,
        RTIOsapiHashAlgorithmKind kind);

extern RTIOsapiDllExport void RTIOsapiHashStream_finalize(
        RTIOsapiHashStream *self);

extern RTIOsapiDllExport RTIBool
RTIOsapiHashStream_hashStart(RTIOsapiHashStream *self);

extern RTIOsapiDllExport RTIBool RTIOsapiHashStream_hashUpdate(
        RTIOsapiHashStream *self,
        const RTI_UINT8 *data,
        size_t size);

extern RTIOsapiDllExport RTIBool
RTIOsapiHashStream_hashFinish(RTIOsapiHashStream *self);

extern RTIOsapiDllExport RTIBool RTIOsapiHashStream_hashFile(
        RTIOsapiHashStream *self,
        const char *fileName);


#ifdef __cplusplus
    }	/* extern "C" */
#endif

#include "osapi/osapi_hash_impl.h"


#endif /* osapi_hash_h */
