/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)osapi_type.h    generated by: makeheader    Sat Mar  9 09:37:10 2024
 *
 *		built from:	type.ifc
 */

#ifndef osapi_type_h
#define osapi_type_h



/* required by size_t */
#include <stddef.h>

#include "osapi/osapi_platform.h"
#include "log/log_common.h"

#ifndef RTIBool
#define RTIBool int
#endif

#ifndef RTI_FALSE
#define RTI_FALSE (0)
#endif

#ifndef RTI_TRUE
#define RTI_TRUE  (1)
#endif

#ifndef NULL
  #define NULL (0)
#endif

/*
 * ALPHA's long is 64 bits
 */
#ifdef RTI_64BIT
    typedef signed long long  RTI_INT64;
    typedef unsigned long long RTI_UINT64;
#elif defined(_WIN32) || defined(_WIN32_WCE)
    typedef __int64 RTI_INT64;
    typedef unsigned __int64 RTI_UINT64;
#elif defined(RTI_SOL2)
    typedef signed long long  RTI_INT64;
    typedef unsigned long long  RTI_UINT64;
#elif defined(RTI_LINUX) || defined(RTI_DARWIN)
    typedef signed long long  RTI_INT64;
    typedef unsigned long long  RTI_UINT64;
#elif defined(RTI_AIX)
  typedef long long  RTI_INT64;
  typedef unsigned long long  RTI_UINT64;
#else 
    typedef long long RTI_INT64;
    typedef unsigned long long RTI_UINT64;
#endif

#if defined(RTI_UNIX) || defined(RTI_VXWORKS) || defined(RTI_OPENVMS)
  /**
    Atomic type that is 32 bits.
  */
    typedef signed int  RTI_INT32;
    typedef unsigned int RTI_UINT32;
  /**
    Atomic type that is 16 bit integer.
  */
    typedef signed short  RTI_INT16;
    typedef unsigned short RTI_UINT16;
  /**
    Atomic type that is 8 bit integer.
  */
    typedef signed char   RTI_INT8;
    typedef unsigned char RTI_UINT8;
  /**
    Atomic type that is 32 bit float.
  */
  typedef float  RTI_FLOAT32;
  /**
    Atomic type that is 64 bit float.
  */
  typedef double RTI_DOUBLE64;
#endif

#if defined(RTI_PSOS)
    typedef signed int     RTI_INT32;
    typedef unsigned int   RTI_UINT32;
    typedef signed short   RTI_INT16;
    typedef unsigned short RTI_UINT16;
    typedef signed char    RTI_INT8;
    typedef unsigned char  RTI_UINT8;
    typedef float          RTI_FLOAT32;
    typedef double         RTI_DOUBLE64;
#endif

#if defined(RTI_NETOS)
    typedef signed int     RTI_INT32;
    typedef unsigned int   RTI_UINT32;
    typedef signed short   RTI_INT16;
    typedef unsigned short RTI_UINT16;
    typedef signed char    RTI_INT8;
    typedef unsigned char  RTI_UINT8;
    typedef float          RTI_FLOAT32;
    typedef double         RTI_DOUBLE64;
#endif

#if defined(RTI_LYNX)
    typedef signed int     RTI_INT32;
    typedef unsigned int   RTI_UINT32;
    typedef signed short   RTI_INT16;
    typedef unsigned short RTI_UINT16;
    typedef signed char    RTI_INT8;
    typedef unsigned char  RTI_UINT8;
    typedef float          RTI_FLOAT32;
    typedef double         RTI_DOUBLE64;
#endif

#if defined(RTI_QNX)
    typedef signed int     RTI_INT32;
    typedef unsigned int   RTI_UINT32;
    typedef signed short   RTI_INT16;
    typedef unsigned short RTI_UINT16;
    typedef signed char    RTI_INT8;
    typedef unsigned char  RTI_UINT8;
    typedef float          RTI_FLOAT32;
    typedef double         RTI_DOUBLE64;
#endif

#if defined(RTI_INTY)
    typedef signed int     RTI_INT32;
    typedef unsigned int   RTI_UINT32;
    typedef signed short   RTI_INT16;
    typedef unsigned short RTI_UINT16;
    typedef signed char    RTI_INT8;
    typedef unsigned char  RTI_UINT8;
    typedef float          RTI_FLOAT32;
    typedef double         RTI_DOUBLE64;
#endif

#ifdef RTI_WIN32
    typedef signed int     RTI_INT32;
    typedef unsigned int   RTI_UINT32;
    typedef signed char    RTI_INT8;
    typedef unsigned char  RTI_UINT8;
    typedef signed short   RTI_INT16;
    typedef unsigned short RTI_UINT16;
    typedef float          RTI_FLOAT32;
    typedef double         RTI_DOUBLE64;
#endif /* RTI_WIN32 */

#if defined(RTI_WINCE)
    typedef int            RTI_INT32;
    typedef unsigned int   RTI_UINT32;
    typedef signed char    RTI_INT8;
    typedef unsigned char  RTI_UINT8;
    typedef short          RTI_INT16;
    typedef unsigned short RTI_UINT16;
    typedef float          RTI_FLOAT32;
    typedef double         RTI_DOUBLE64;
#endif /* RTI_WINCE */

#ifdef RTI_INTIME
    typedef signed int     RTI_INT32;
    typedef unsigned int   RTI_UINT32;
    typedef signed char    RTI_INT8;
    typedef unsigned char  RTI_UINT8;
    typedef signed short   RTI_INT16;
    typedef unsigned short RTI_UINT16;
    typedef float          RTI_FLOAT32;
    typedef double         RTI_DOUBLE64;
#endif

#if defined(RTI_WISP)
    typedef signed int     RTI_INT32;
    typedef unsigned int   RTI_UINT32;
    typedef double         RTI_DOUBLE64;
#endif

/* This definition is kept here to guarantee backward compatibility if the
 * user is already using RTI_CDR_SIZEOF_LONG_DOUBLE to activate native
 * long double support
 */
#if (RTI_CDR_SIZEOF_LONG_DOUBLE == 16)
  #ifndef RTI_OSAPI_SIZEOF_LONG_DOUBLE
    #define RTI_OSAPI_SIZEOF_LONG_DOUBLE 16
  #endif
#endif

#if (RTI_OSAPI_SIZEOF_LONG_DOUBLE == 16)
  typedef long double RTI_DOUBLE128;
#else
  typedef struct RTI_DOUBLE128 {
      char bytes[16];
  } RTI_DOUBLE128;
#endif /* RTI_OSAPI_SIZEOF_LONG_DOUBLE == 16 */


#define RTI_UINT8_MAX 255
#define RTI_UINT8_MIN 0

#define RTI_INT16_MAX 32767
#define RTI_INT16_MIN (-32767 - 1)

#define RTI_UINT16_MAX 65535
#define RTI_UINT16_MIN 0

/* This is not defined as -2147483648 because that value is processed in
 two stages. From https://msdn.microsoft.com/en-us/library/4kh09110.aspx
 1. The number 2147483648 is evaluated. Because it is greater than the                                                                                         .
    maximum integer value of 2147483647, the type of 2147483648 is not int,                                                                                    .
    but unsigned int                                                                                                                                           .
 2. Unary minus is applied to the value, with an unsigned result, which also                                                                                                                                                                                        .
    happens to be 2147483648.                                                                                                                                                              .
*/
#define RTI_INT32_MAX 2147483647
#define RTI_INT32_MIN (-2147483647 - 1)

#define RTI_UINT32_MAX 4294967295U
#define RTI_UINT32_MIN 0

#define RTI_INT64_MAX  9223372036854775807LL
#define RTI_INT64_MIN  (-RTI_INT64_MAX - 1LL)

#define RTI_UINT64_MAX 18446744073709551615ULL
#define RTI_UINT64_MIN 0ULL

#define RTI_FLOAT_MAX  3.40282346638528859811704183484516925e+38F
#define RTI_FLOAT_MIN  (-RTI_FLOAT_MAX)

#define RTI_DOUBLE_MAX  ((double)1.79769313486231570814527423731704357e+308L)
#define RTI_DOUBLE_MIN  (-RTI_DOUBLE_MAX)

/*
 * The following value is the maximum value that can be converted into a double
 * without losing precision. And it is calculated as follow:
 *   pow(2, DBL_MANT_DIG -1) where DBL_MANT_DIG = 53
 */
#define RTI_DOUBLE_MANT_MAX (4503599627370496.0)
#define RTI_DOUBLE_MANT_MAX_AS_LONG_LONG (4503599627370496LL)

#ifdef RTI_64BIT
  #define RTI_SIZE_MAX RTI_UINT64_MAX
#else
  #define RTI_SIZE_MAX RTI_UINT32_MAX
#endif
#define RTI_SIZE_MIN 0

/**
 * @brief Unlimited value for 32-bit signed integers.
 */
#define RTI_INT32_SIZE_UNLIMITED -1
/**
 * @brief Unlimited value for 64-bit signed integers.
 */
#define RTI_INT64_SIZE_UNLIMITED -1

/*e
 * \ingroup RTIOsapiHeapClass
 * @brief Generic buffer to hold data, described by a pointer to the data, as
 * well as the length of the data.
 *
 * This structure deliberately mirrors
 * windows WSABUF so we can cast the passed in structure directly without
 * copying the fields into a structure on the stack.
 */

struct RTIBuffer {
    /*e Length of the buffer pointed to by pointer. */
    RTI_INT32 length;
    /*e Pre-allocated (by the caller) buffer.  */
    char *pointer;
};


#ifdef __cplusplus
    extern "C" {
#endif


#ifdef __cplusplus
    }	/* extern "C" */
#endif

#endif /* osapi_type_h */
