/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)osapi_utility.h    generated by: makeheader    Sat Mar  9 09:37:10 2024
 *
 *		built from:	utility.ifc
 */

#ifndef osapi_utility_h
#define osapi_utility_h



#include <errno.h>

#if defined(RTI_WIN32) || defined(RTI_WINCE)
  #ifndef WIN32_LEAN_AND_MEAN
    #define WIN32_LEAN_AND_MEAN
  #endif
  #include <windows.h>
  #include <errhandlingapi.h>
#endif

#include <float.h>
#include "osapi/osapi_type.h"
#include "osapi/osapi_dll.h"
#include "osapi/osapi_vxVersions.h"
#include "osapi/osapi_rtpsTime.h"
#include "osapi/osapi_file.h"

#include <stdio.h>
#include <stdarg.h>

#if defined(RTI_INTY)
  #include <util/error_string.h>
#endif

#if (defined(RTI_VXWORKS) || defined(RTI_QNX) || defined(RTI_INTY))
  #define RTI_DOUBLE_SIGNIFICANT_DIGITS   15
#else
  #define RTI_DOUBLE_SIGNIFICANT_DIGITS   17
#endif
#define RTI_FLOAT_SIGNIFICANT_DIGITS   9

#ifdef RTI_WIN32
    #define RTI_OSAPI_FILE_PATH_SEPARATOR ("\\")
#else
    #define RTI_OSAPI_FILE_PATH_SEPARATOR ("/")
#endif


#define RTI_OSAPI_PATH_MAX_SIZE 512


#if !(defined(RTI_VXWORKS) && VXWORKS_MAJOR_VERSION < 6) \
        && !(defined(RTI_LYNX) && RTI_LYNX < 500)
  #define RTI_OSAPI_VSNPRINTF_SUPPORTED
#endif


#ifdef __cplusplus
    extern "C" {
#endif

extern RTIOsapiDllExport int RTIOsapiUtility_max(int l, int r);

extern RTIOsapiDllExport int RTIOsapiUtility_isWithinBound(
    int me, int lower, int upper);

extern RTIOsapiDllExport int RTIOsapiUtility_min(int l, int r);

extern RTIOsapiDllExport int RTIOsapiUtility_abs(int l);

extern RTIOsapiDllExport RTI_UINT16 RTIOsapiUtility_ntohs(RTI_UINT16 s);

extern RTIOsapiDllExport RTI_UINT32 RTIOsapiUtility_ntohl(RTI_UINT32 l);

extern RTIOsapiDllExport RTI_UINT16 RTIOsapiUtility_htons(RTI_UINT16 s);

extern RTIOsapiDllExport RTI_UINT32 RTIOsapiUtility_htonl(RTI_UINT32 l);

extern RTIOsapiDllExport 
void RTIOsapiUtility_hostToNetworkByteOrder(
    void *dst, 
    const void *src,
    RTI_UINT32 size);

extern RTIOsapiDllExport int RTIOsapiUtility_isDigit(int c);


#define OSAPI_UTILITY_MAX_TEMP_DIRECTORY_NAME_LENGTH 256

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_createTempDirectory(char *outPath);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_deleteDirectory(const char *path);

extern RTIOsapiDllExport 
char* RTIOsapi_envVarOrFileGet(char *buffer, int bufferLen,
                                 const char *envVariableName,
                                 const char *fileName,
                                 char separator, 
                                 char commentchar);

extern RTIOsapiDllExport RTIBool
RTIOsapiUtility_getUsername(char *username, size_t bufferSize);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_getExecutablePath(
        char *path,
        int bufferSize,
        const char *executable);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_isGetFilePathSupported(void);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_getFilePath(
        char *path,
        size_t pathMaxSize,
        const char *file);

 
extern	RTIOsapiDllExport RTIBool RTIOsapiUtility_getSelfFullpath(
        char *path, 
        size_t bufferSize);

extern RTIOsapiDllExport RTIBool
RTIOsapiUtility_getSelfDirectoryPath(char *path, size_t bufferSize);

extern RTIOsapiDllExport
char *RTIOsapiUtility_getCurrentDirectory(char *path, size_t bufferSize);

extern RTIOsapiDllExport RTIBool
RTIOsapiUtility_getTarget(char *target, size_t bufferSize);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_getCreationTimestamp(char *buf, int bufferSize);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_getExecutionTimestamp(char *buf, int bufferSize);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_normalizePath(
        char *path,
        size_t bufferSize,
        const char *originalPath);


#if defined(RTI_IOS_xxx)    \
    || defined(RTI_LINUX)   \
    || defined(RTI_DARWIN)  \
    || defined(RTI_VXWORKS) \
    || defined(RTI_AIX)     \
    || defined(RTI_INTY)
  #define RTI_OSAPI_SETENV_SUPPORTED
#endif

#if defined(RTI_WIN32)   \
    || defined(RTI_INTY) \
    || defined(RTI_QNX)  \
    || defined(RTI_OSAPI_SETENV_SUPPORTED)
  #define RTI_OSAPI_ENVIRONMENT_SET_SUPPORTED
#endif


extern RTIOsapiDllExport
RTIBool RTIOsapi_isEnvVarSetSupported(void);

extern RTIOsapiDllExport
RTIBool RTIOsapi_envVarSet(const char * envVariable);

extern RTIOsapiDllExport
RTIBool RTIOsapi_envVarUnset(const char * varName);

extern RTIOsapiDllExport
int RTIOsapiUtility_snprintf(char *str, size_t size, const char *format, ...);

extern RTIOsapiDllExport
int RTIOsapiUtility_snprintfAdv(char *str, size_t size, RTIBool showErrors, const char *format, ...);

extern RTIOsapiDllExport
int RTIOsapiUtility_vsnprintf(char *str, size_t size, const char *format, va_list args);

extern RTIOsapiDllExport
int RTIOsapiUtility_vsnprintfAdv(char *str, size_t size, RTIBool showErrors, const char *format, va_list args);

extern RTIOsapiDllExport
int RTIOsapiUtility_sscanf(const char *s, const char *format, ...);

extern RTIOsapiDllExport
size_t RTIOsapiUtility_strlen(const char *str);

extern RTIOsapiDllExport 
size_t RTIOsapiUtility_strnlen(const char *str, size_t maxLength);

extern RTIOsapiDllExport
size_t RTIOsapiUtility_strlenFnc(char *str);

extern RTIOsapiDllExport
char *RTIOsapiUtility_strcpy(char *dest, size_t destStringLengthMax, const char *src);

extern RTIOsapiDllExport char *RTIOsapiUtility_strncpy(
        char *dest,
        size_t destStringLengthMax,
        const char *src,
        size_t n);

extern RTIOsapiDllExport char *RTIOsapiUtility_strcat(
        char *dest,
        size_t destStringLengthMax,
        const char *src);

extern RTIOsapiDllExport char *RTIOsapiUtility_strncat(
        char *dest,
        size_t destStringLengthMax,
        const char *src,
        size_t n);

extern RTIOsapiDllExport
const char *RTIOsapiUtility_strstr(const char *src, const char *pattern);

extern RTIOsapiDllExport
RTI_INT32 RTIOsapiUtility_strncmp(
        const char *str1,
        const char *str2,
        size_t size);

extern RTIOsapiDllExport
RTI_INT32 RTIOsapiUtility_strcmp(
        const char *str1,
        const char *str2);

extern RTIOsapiDllExport
int RTIOsapiUtility_strcasecmp(const char *src, const char *pattern);

extern RTIOsapiDllExport
unsigned int RTIOsapiUtility_get_double_precision(void);

extern RTIOsapiDllExport RTIBool RTIOsapiUtility_strtolWithBase(
        const char *str,
        const char **endPtr,
        RTI_INT32 *value,
        RTI_INT32 base);

extern RTIOsapiDllExport RTIBool
RTIOsapiUtility_strtol(const char *str, RTI_INT32 *value);

extern RTIOsapiDllExport RTIBool RTIOsapiUtility_strtoulWithBase(
        const char *str,
        const char **endPtr,
        RTI_UINT32 *value,
        RTI_INT32 base);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_strtoul(const char * str, RTI_UINT32 * value);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_strtoll(const char * str, RTI_INT64 * value);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_strtollWithEndSeparator(
        const char *str,
        RTI_INT64 *value,
        const char expectedEndSeparator);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_strtoull(const char * str, RTI_UINT64 * value, int base);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_strtod(const char *str, RTI_DOUBLE64 *value);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_parseHexString(
        const char *strIn,
        unsigned char *bufOut,
        size_t maxSize,
        size_t *bufSizeOut);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_strtoptr(const char * str,void ** value);


#define RTI_OSAPI_STRING_SEQ_STRING_MAX_SIZE (4096)

#if defined(RTI_VXWORKS) && VXWORKS_VERSION_6_3_OR_BETTER \
    || defined(RTI_VX653) \
    || (ENABLE_FACE_COMPLIANCE != FACE_COMPLIANCE_LEVEL_NONE)
  #define RTI_REALTIME_CLOCK_ID CLOCK_REALTIME
#endif

extern RTIOsapiDllExport
RTI_INT64 RTIOsapiUtility_gethrtime(void);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_getTimeAdv(
        struct RTINtpTime *now,
        RTIBool logException);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_getTime(struct RTINtpTime *now);


#define RTI_OSAPI_UTC_TIME_BUFFER_SIZE 29

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_getUtcTime(
        char *buffer,
        unsigned int size,
        const struct RTINtpTime *time,
        RTIBool includeDate,
        RTIBool includeMicroSec,
        RTIBool includeSeparators);

extern RTIOsapiDllExport
void RTIOsapiUtility_getUtcTimeForLogging(
        char *buffer,
        unsigned int size,
        const struct RTINtpTime *time);

extern RTIOsapiDllExport
char * RTIOsapiUtility_strToken(char *s1, const char *delim, char **save);

extern RTIOsapiDllExport
char *  RTIOsapiUtility_strTokenEx(
        char *s1,
        const char *delim,
        const char *escapeAround,
        const char *escapeBefore,
        char **save);

extern RTIOsapiDllExport
unsigned int RTIOsapiUtility_pow2(int y);

extern RTIOsapiDllExport
unsigned int RTIOsapiUtility_log2(unsigned int x, char floor);

extern RTIOsapiDllExport
char * RTIOsapiUtility_strTrimStart(char *str);

extern RTIOsapiDllExport
char * RTIOsapiUtility_strTrimStartEx(char *str, const char *charactersToSkip);

extern RTIOsapiDllExport char *RTIOsapiUtility_strTrim(char *str);

extern RTIOsapiDllExport
char * RTIOsapiUtility_strReplaceString(
        char *str,
        const char *strToReplace,
        const char *strReplacement);

extern RTIOsapiDllExport
void RTIOsapiUtility_srandWithSeed(unsigned int seed);

extern RTIOsapiDllExport
void RTIOsapiUtility_srand(void);

extern RTIOsapiDllExport
int RTIOsapiUtility_rand(void);


#define RTI_OSAPI_UTILITY_RETCODE_OK 0
#define RTI_OSAPI_UTILITY_RETCODE_ERROR -1
#define RTI_OSAPI_UTILITY_RETCODE_NOT_SUPPORTED -2

extern RTIOsapiDllExport int RTIOsapiUtility_randSecure(unsigned int *buffer);

extern RTIOsapiDllExport
int RTIOsapiUtility_randRange(int rangeMin, int rangeMax);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_isnanf(float x);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_isnan(double x);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_isinff(float x);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_isinf(double x);


#define RTIOsapiUtility_getEnabledBitCount RTILog_getEnabledBitCount

extern RTIOsapiDllExport
unsigned int RTIOsapiUtility_getLeastSignificantEnabledBit(unsigned int i);

extern RTIOsapiDllExport
int RTIOsapiUtility_linearNormalization(
        int x,
        int rMin,
        int rMax,
        int targetMin,
        int targetMax);


#define RTI_OSAPI_ERROR_STRING_MAX_SIZE (128)
#define RTI_OSAPI_UNKNOWN_ERROR_STRING "Unknown error"


#if defined(RTI_WIN32) || defined(RTI_WINCE)
  #define RTIOsapiUtility_getError GetLastError
  #define RTIOsapiUtility_setError SetLastError
#else
  #ifdef RTI_VXWORKS
    #ifndef errno
      #define errno errnoGet()
    #endif
  #endif
  #define RTIOsapiUtility_getError() errno
  #define RTIOsapiUtility_setError(x) errno = (x)
#endif

#define RTIOsapiUtility_reportFileErrno( \
        instrumentBit__, \
        logTemplate__, \
        fileName__) \
  RTIOsapiUtility_reportFileErrnoI( \
          RTI_FUNCTION_NAME, \
          instrumentBit__, \
          logTemplate__, \
          fileName__)


extern RTIOsapiDllExport
const char * RTIOsapiUtility_getErrorString(
    char *buffer, size_t bufferSize, int errnum);

extern RTIOsapiDllExport void RTIOsapiUtility_reportFileErrnoI(
        const char *functionName,
        RTILogBitmap instrumentBit,
        const struct RTILogMessage logTemplate,
        const char *fileName);

extern RTIOsapiDllExport
void * RTIOsapiUtility_intToPointer(const RTI_INT32 intValue);

extern RTIOsapiDllExport
RTI_INT32 RTIOsapiUtility_pointerToInt(const void *pointer);

extern RTIOsapiDllExport
RTI_UINT64 RTIOsapiUtility_pointerToUInt64(const void *pointer);

extern RTIOsapiDllExport
RTI_UINT32 RTIOsapiUtility_crc32(
        const void *data,
        RTI_UINT32 length,
        RTI_UINT32 previousCrc32);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_isNativeLittleEndian(void);

extern RTIOsapiDllExport 
RTIBool RTIOsapiUtility_stringArrayToDelimitedString(
        char* dst,
        size_t dst_max,
        char delim,
        const char **stringArray,
        int stringArrayLength);

extern RTIOsapiDllExport
int RTIOsapiUtility_stringUtf8ValidateChar(const char *inputString);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_stringUtf8ValidateString(const char *inputString);

extern RTIOsapiDllExport
int RTIOsapiUtility_stringUtf8ToCodepoint(
        RTI_UINT32 *codepoint,
        const char *inputString);

extern RTIOsapiDllExport
int RTIOsapiUtility_stringUtf8FromCodepoint(
        char *outputString,
        const RTI_UINT32 outputStringLength,
        const RTI_UINT32 codepoint);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_stringUtf16ToSurrogatePair(
        RTI_UINT16 *highSurrogate,
        RTI_UINT16 *lowSurrogate,
        const RTI_UINT32 codepoint);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_stringUtf16FromSurrogatePair(
        RTI_UINT32 *codepoint,
        const RTI_UINT16 highSurrogate,
        const RTI_UINT16 lowSurrogate);

extern RTIOsapiDllExport
int RTIOsapiUtility_stringUtf8ToUtf16(
        RTI_UINT16 *outputString,
        RTI_UINT32 outputStringLength,
        const char *inputString);

extern RTIOsapiDllExport
int RTIOsapiUtility_stringUtf16ToUtf8(
        char *outputString,
        RTI_UINT32 outputStringLength,
        const RTI_UINT16 *inputString,
        RTI_UINT32 inputStringLength);

extern RTIOsapiDllExport
const char * RTIOsapiUtility_strGetToken(
        size_t *tokenLength,
        const char *s,
        char c);

extern RTIOsapiDllExport
size_t RTIOsapiUtility_strGetTokenCount(const char *string, char separator);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_sizeToInt32(RTI_INT32 dst, size_t src);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_sizeToUInt32(RTI_UINT32 dst, size_t src);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_sizeToInt64(RTI_UINT64 dst, size_t src);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_int32ToSize(size_t dst, RTI_INT32 src);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_int64ToSize(size_t dst, RTI_INT64 src);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_uint64ToSize(size_t dst, RTI_UINT64 src);

extern RTIOsapiDllExport RTIBool
RTIOsapiUtility_floatNearlyEqual(float first, float second);

extern RTIOsapiDllExport RTIBool
RTIOsapiUtility_doubleNearlyEqual(double first, double second);

extern RTIOsapiDllExport
RTIBool RTIOsapiUtility_int64ToDouble(RTI_DOUBLE64 *dst, RTI_INT64 src);


/*
 * @brief Enables developers to instrument the execution time of a code block.
 */
#define RTIOsapiUtility_timeInstrumentationStart() \
{ \
    struct RTINtpTime startTime__ = RTI_NTP_TIME_ZERO; \
    struct RTINtpTime endTime__ = RTI_NTP_TIME_ZERO; \
    struct RTINtpTime elapsedTime__ = RTI_NTP_TIME_ZERO; \
    unsigned int totalTimeMicroSec__ = 0; \
    int sec__ = 0; \
    unsigned int microsec__ = 0; \
    \
    if (!RTIOsapiUtility_getTime(&startTime__)) { \
        RTILogMessage_print( \
                RTI_LOG_PRINT_FORMAT_MASK_ALL, \
                RTI_LOG_BIT_EXCEPTION, \
                MODULE_OSAPI, \
                &RTI_LOG_GET_FAILURE_s, \
                "current time"); \
    } \

#define RTIOsapiUtility_timeInstrumentationEnd(instrumented_block_name__) \
    if (!RTIOsapiUtility_getTime(&endTime__)) { \
        RTILogMessage_print( \
                RTI_LOG_PRINT_FORMAT_MASK_ALL, \
                RTI_LOG_BIT_EXCEPTION, \
                MODULE_OSAPI, \
                &RTI_LOG_GET_FAILURE_s, \
                "current time"); \
    } \
    \
    RTINtpTime_subtract(elapsedTime__, endTime__, startTime__); \
    \
    RTINtpTime_unpackToMicrosec(sec__, microsec__, elapsedTime__); \
    totalTimeMicroSec__ = sec__ * 1000000 + microsec__; \
    RTILogParamString_printPlain( \
            "%s took %d us\n", \
            (instrumented_block_name__), \
            totalTimeMicroSec__); \
} \

/**
 * TRUST-756: Helper macro to prevent a pointer value from being speculated.
 * It addresses the Spectre Variant 1 vulnerability in a platform independent way.
 * The technique that we employ here is called Speculative Load Hardening.
 *
 * The events contributing to speculative execution in coverity typically start
 * with an if/else/for/while block.
 * Use this macro inside the block if the true branch is taken otherwise
 * use it just outside the block by passing the condition by prepending it with !.
 *
 * For reference coverity will report something like this as the first
 * contributing event for the issue:
 *      compare_value: Comparing <condition>
 *      Condition <condition>, taking true/false branch
 * If the true branch is taken you would pass <condition> to this macro while
 * using it inside the block.
 * If the false branch is taken you would pass !<condition> to this macro while
 * using it outside the block.
 *
 * @brief
 * The basic idea to avoid speculative execution of an if/else/for/while block
 * is to force the entry condition execution using a ternary operator. A ternary
 * operator (or a conditional update) based assignment is guaranteed to be
 * implemented in a branchless way on most modern architectures:
 * https://llvm.org/docs/SpeculativeLoadHardening.html#high-level-mitigation-approach
 *
 * @details
 * The AUDIT.SPECULATIVE_EXECUTION_DATA_LEAK checker from Coverity detects
 * Spectre Variant 1 vulnerabilities. This is detailed in this blog post:
 * https://www.synopsys.com/blogs/software-security/detecting-spectre-vulnerability-exploits-with-static-analysis.html
 *
 * This macro adopts the mitigation approach detailed in this post for Variant 1:
 * https://llvm.org/docs/SpeculativeLoadHardening.html#high-level-mitigation-approach
 *
 * For more information about Spectre and Speculative Execution you can also read the paper:
 * https://spectreattack.com/spectre.pdf
 *
 *
 * @param pointer__
 * The pointer to harden against speculative execution.
 * This is typically reported by coverity under the second event contributing to
 * the issue. It should be shown as 'compared_memory_access'.
 *
 * @param pointerType__
 * The type used to cast the result from the ternary operator (conditional update).
 *
 * @param speculatedCondition__
 * The condition that may be bypassed by speculation. For an if/else if/for/while
 * block copy the same condition as is. For the else block copy the if condition
 * by prefixing it with a !.
 * This is typically reported by coverity as the first event contributing
 * to the issue. It should be shown as 'compare_value'.
 *
 */
#define RTIOsapiUtility_preventPointerSpeculation( \
        pointer__, \
        pointerType__, \
        speculatedCondition__) \
{ \
    const size_t RTI_OSAPI_UTILITY_ALL_BITS_ZERO = 0; \
    const size_t RTI_OSAPI_UTILITY_ALL_BITS_ONE = ~(RTI_OSAPI_UTILITY_ALL_BITS_ZERO); \
    size_t pointerMask = RTI_OSAPI_UTILITY_ALL_BITS_ONE; \
    \
    pointerMask = (!(speculatedCondition__)) \
            ? RTI_OSAPI_UTILITY_ALL_BITS_ZERO \
            : pointerMask; \
    (pointer__) = (pointerType__) ((size_t) (pointer__) & pointerMask); \
}



#ifdef __cplusplus
    }	/* extern "C" */
#endif

  #include "osapi/osapi_utility_impl.h"

#endif /* osapi_utility_h */
