/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)reda_bitVector_impl.h    generated by: makeheader    Sat Mar  9 09:37:38 2024
 *
 *		built from:	bitVector_impl.ifc
 */

#ifndef reda_bitVector_impl_h
#define reda_bitVector_impl_h



#include "limits.h"

  #ifndef reda_dll_h
    #include "reda/reda_dll.h"
  #endif
  #ifndef osapi_type_h
    #include "osapi/osapi_type.h"
  #endif
  #ifndef osapi_alignment_h
    #include "osapi/osapi_alignment.h"
  #endif

#ifdef __cplusplus
    extern "C" {
#endif


struct REDABitVector {
    RTI_UINT8 *_bitfield;
    RTI_UINT32 _bitfieldSize;
    RTI_INT32 _bitfieldSizeInBytes;
};



/** \ingroup REDABitVectorClass
 *
 * @brief Returns a bit within the BitVector
 *
 * @param bitPosition__
 * The requested bit position of the BitVector starting by 0
 *
 * @return
 * The value of the bit of type RTI_UINT8.
 */
#define REDABitVector_getBit(selfPointer__, bitPosition__) \
    (((selfPointer__)->_bitfield[(bitPosition__) / 8] >> ((bitPosition__) % 8)) \
            & 0x01)


/** \ingroup REDABitVectorClass
 *
 * @brief Set a bit within the BitVector to 1
 *
 * @param bitPosition__
 * The requested bit position of the BitVector starting by 0
 *
 */
#define REDABitVector_setBitToOne(selfPointer__, bitPosition__) \
    (selfPointer__)->_bitfield[(bitPosition__) / 8] |= \
            (RTI_UINT8) (((RTI_UINT32) 1) << ((bitPosition__) % 8))

/** \ingroup REDABitVectorClass
 *
 * @brief Set a bit within the BitVector to zero
 *
 * @param bitPosition__
 * The requested bit postion of the BitVector starting by 0
 *
 */
#define REDABitVector_setBitToZero(selfPointer__, bitPosition__) \
    (selfPointer__)->_bitfield[(bitPosition__) / 8] &= \
            (RTI_UINT8) ~(((RTI_UINT32) 1) << ((bitPosition__) % 8))

/** \ingroup REDABitVectorClass
 *
 * @brief Set a bit within the BitVector to to the specified value.
 *
 * This operation will mask to the least significant bit of the bit value.
 *
 * @param bitPosition__
 * The requested bit postion of the BitVector starting by 0
 *
 * @param bitValue__
 * The value of the bit. Either 1 or 0.
 *
 */
#define REDABitVector_setBit(selfPointer__, bitPosition__, bitValue__) { \
    int bytePosition = (bitPosition__) / 8; \
    int relativeBitPosition = (bitPosition__) % 8; \
    RTI_UINT8 currentByteWithBitReset = 0; \
    \
    /* Clear the bit value first */ \
    currentByteWithBitReset = (selfPointer__)->_bitfield[bytePosition] \
            & (RTI_UINT8) ~(((RTI_UINT32) 1) << relativeBitPosition); \
    /* Now set the new value using bitwise OR operator */ \
    (selfPointer__)->_bitfield[bytePosition] = currentByteWithBitReset \
            | (RTI_UINT8) (((RTI_UINT32) (bitValue__) && 0x01) << relativeBitPosition); \
}

/** \ingroup REDABitVectorClass
 *
 * @brief Returns the underlying array of type RTI_UINT8.
 *
 * @param selfPointer__
 * Pointer to the REDABitVector.
 */
#define REDABitVector_getCharArray(selfPointer__) (selfPointer__)->_bitfield



#ifdef __cplusplus
    }	/* extern "C" */
#endif


#endif /* reda_bitVector_impl_h */
