/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)reda_skiplist_impl.h    generated by: makeheader    Sat Mar  9 09:37:42 2024
 *
 *		built from:	skiplist_impl.ifc
 */

#ifndef reda_skiplist_impl_h
#define reda_skiplist_impl_h


#ifdef __cplusplus
    extern "C" {
#endif


#define REDA_SKIPLIST_MAGIC_NUMBER 0x7344L

struct REDASkiplistNode {
    /*i userData (key and data); note that the compare function
      is invoked on this userData. 
    */
    void *userData;
    /*i level is in [0, list->_desc->maximumLevel]. */
    char level;
    /*i userChar */
    char userChar;
    /*i userInt */
    int userInt;
    /*i the only backward pointer; enables reverse iteration */
    struct REDASkiplistNode *backward;
    /*i the first forward pointer; this is always present, and should NOT be
      moved around because of the reason given shortly. */
    struct REDASkiplistNode *forward;
    /* more forward pointers are stored here so that they can
       be accessed as (&forward)[level]. */
};

struct REDASkiplistDescription {
    /*e Node creation function */
    REDASkiplistNodeCreateFunction nodeCreateFnc;
    /*e Parameter to REDASkiplistNodeCreateFunction */
    void *nodeCreateParam;
    /*e Node destruction function */
    REDASkiplistNodeDestroyFunction nodeDestroyFnc;
    /*e Parameter to REDASkiplistNodeDestroyFunction */
    void *nodeDestroyParam;
    /*e Function to create array of forward pointers for a node */
    REDASkiplistNodeLevelGenerateFunction nodeLevelGenerateFnc;
    /*e Parameter to REDASkiplistNodeLevelGenerateFunction */
    void *nodeLevelGenerateParam;
    /*e Maximum level for any nodes in the skiplist */
    char maximumLevel;
};


#define REDA_SKIPLIST_DESCRIPTION_INITIALIZER \
{ \
    NULL, /* nodeCreateFnc */ \
    NULL, /* nodeCreateParam */ \
    NULL, /* nodeDestroyFnc */ \
    NULL, /* nodeDestroyParam */ \
    NULL, /* nodeLevelGenerateFnc */ \
    NULL, /* nodeLevelGenerateParam */ \
    0 /* maximumLevel */ \
}


  #define REDASkiplistNode_getForwardPointer(node, level) \
    ( (&((node)->forward))[(level)] )

struct REDASkiplist {
    /* A magic number that serves as a hint as to whether this
       skiplist has been initialized yet*/     
    int _skiplist_init;
    /*i number of nodes in the list.  */
    int _nodeCount;
    /*i head can skip forward at any valid level. */
    struct REDASkiplistNode *_head;
    /*i the last node in the list */
    struct REDASkiplistNode *_last;
    /*i Description of the skiplist */
    const struct REDASkiplistDescription *_desc;
    /*e Function to compare two userDatas */
    REDAOrderedDataTypeCompareFunction nodeKeyCompareFnc;
    /*i _currentMaxLevel is in [0, _desc->maximumLevel] */
    char _currentMaxLevel;
    /*i _Function to compare two node keys and also to pass addition userdata */
    REDAOrderedDataTypeCompareWithUserDataFunction nodeKeyCompareWithUserDataFnc;
    /*i _Any additional user data to  passed to the copare function*/
    void * userData;

    unsigned int _cumulativeNodeCount;
    RTIBool _isBuiltinCompareFcn;
};


#define REDA_SKIPLIST_INITIALIZER \
{ \
    0, /* _skiplist_init */ \
    0, /* _nodeCount */ \
    NULL, /* _head */ \
    NULL, /* _last */ \
    NULL, /* _desc */ \
    NULL, /* nodeKeyCompareFnc */ \
    0, /* _currentMaxLevel */ \
    NULL, /* nodeKeyCompareWithUserDataFnc */ \
    NULL, /* userData */ \
    0, /* _cumulativeNodeCount */ \
    RTI_FALSE /* _isBuiltinCompareFcn */ \
}


  #define REDASkiplist_forwPtrCount(level) (level+1)


  #define REDA_SKIPLIST_FORW_PTR_MAX  \
      REDASkiplist_forwPtrCount(REDA_SKIPLIST_LEVEL_MAX_UPPER_LIMIT)


  #define REDASkiplist_sizeOfSkiplistNode(level) \
    (sizeof(struct REDASkiplistNode) + \
     (size_t) (level) * sizeof(struct REDASkiplistNode *))


  #ifndef RTI_PRECONDITION_TEST
    #define REDASkiplist_gotoTopNode    REDASkiplist_gotoTopNodeMacro
    #define REDASkiplist_gotoLastNode    REDASkiplist_gotoLastNodeMacro
    #define REDASkiplist_gotoNextNode   REDASkiplist_gotoNextNodeMacro
    #define REDASkiplist_gotoPreviousNode   REDASkiplist_gotoPreviousNodeMacro
    #define REDASkiplist_getNodeCount   REDASkiplist_getNodeCountMacro
    #define REDASkiplist_getLevel       REDASkiplist_getLevelMacro
    #define REDASkiplist_isInitialized  REDASkiplist_isInitializedMacro
    #define REDASkiplist_findNode(self, answer, preciseMatch, key) \
        REDASkiplist_findNodeFastMacro(self, answer, preciseMatch, key)
  #endif /* !RTI_PRECONDITION_TEST */
/* ---------------------------------------------------------------------- */
/*i @note NEVER precondition tested! */
  #define REDASkiplistNode_getUserData(node) (node)->userData
/* ---------------------------------------------------------------------- */
  #define REDASkiplistNode_getUserChar(node) (node)->userChar
/* ---------------------------------------------------------------------- */
  #define REDASkiplistNode_getUserInt(node) (node)->userInt
/* ---------------------------------------------------------------------- */
  #define REDASkiplist_gotoTopNodeMacro(self, node) \
    *(node) = (self)->_head
/* ---------------------------------------------------------------------- */
  #define REDASkiplist_gotoLastNodeMacro(self, node) \
    ((*(node) = (self)->_last) != NULL)
/* ---------------------------------------------------------------------- */
  #define REDASkiplist_gotoNextNodeMacro(self, node) \
    ( (*(node) = (*(node))->forward) != NULL )
/* ---------------------------------------------------------------------- */
  #define REDASkiplist_gotoPreviousNodeMacro(self, node) \
    ( (*(node) = (*(node))->backward) != (self)->_head && *(node) != NULL)
/* ---------------------------------------------------------------------- */
  #define REDASkiplist_getNodeCountMacro(self) (self)->_nodeCount
/* ---------------------------------------------------------------------- */
  #define REDASkiplist_getLevelMacro(self) (self)->_currentMaxLevel
/* ---------------------------------------------------------------------- */
  #define REDASkiplist_isInitializedMacro(self)  \
    (((self)->_skiplist_init == REDA_SKIPLIST_MAGIC_NUMBER)?RTI_TRUE:RTI_FALSE)


#define REDASkiplist_findNodeWithUpdateMacro(self, answer, preciseMatch, \
                                             searchKey, update)          \
{                                                                        \
    register int level = (int)((self)->_currentMaxLevel);                \
    register int compResult = -1;                                        \
    register struct REDASkiplistNode *nextNode = NULL;                   \
    register struct REDASkiplistNode *currNode;                          \
    register void *inKey = (searchKey);                                  \
    register void *__userData = (self)->userData;                          \
    register struct REDASkiplistNode **updateArg = (update);             \
    register REDAOrderedDataTypeCompareFunction compareFnc =             \
        (self)->nodeKeyCompareFnc;                                       \
    register REDAOrderedDataTypeCompareWithUserDataFunction compareFncWithUserData =            \
        (self)->nodeKeyCompareWithUserDataFnc;                           \
                                                                         \
    for (currNode = (self)->_head; level >= 0; --level) {                \
	while (((nextNode =                                              \
                 REDASkiplistNode_getForwardPointer(currNode,            \
                                                    level)) != NULL)     \
               && (nextNode->userData != NULL)                           \
               && ((compResult = ((self)->_isBuiltinCompareFcn) ?        \
                                REDAOrderedDataType_compareUsingFnc(compareFnc,         \
                                                                    nextNode->userData, \
                                                                    inKey):             \
                                (compareFncWithUserData ?                                       \
                                compareFncWithUserData(nextNode->userData,inKey,__userData) :     \
                                compareFnc(nextNode->userData,inKey))) < 0) ) {                 \
	    currNode = nextNode;                                         \
        }                                                                \
        updateArg[level] = currNode;                                     \
    }                                                                    \
    if ((void *) preciseMatch != NULL) {                                 \
	*(preciseMatch) = (compResult == 0);                             \
    }                                                                    \
    *(answer) = nextNode;                                                \
}


#define REDASkiplist_findLastNodeWithUpdateMacro(self, answer, update)   \
{                                                                        \
    register int level = (int)((self)->_currentMaxLevel);                \
    register struct REDASkiplistNode *nextNode = NULL;                   \
    register struct REDASkiplistNode *currNode;                          \
    register struct REDASkiplistNode **updateArg = (update);             \
                                                                         \
    for (currNode = (self)->_head; level >= 0; --level) {                \
	while (((nextNode =                                              \
                 REDASkiplistNode_getForwardPointer(currNode,            \
                                                    level)) != NULL)) {  \
	    currNode = nextNode;                                         \
        }                                                                \
        updateArg[level] = currNode;                                     \
    }                                                                    \
    *(answer) = nextNode;                                                \
}


#define REDASkiplist_findNodeFastMacro(                              \
        self__,                                                      \
        answer__,                                                    \
        preciseMatch__,                                              \
        searchKey__)                                                 \
{                                                                    \
    register int level__;                                            \
    register int compResult__ = -1; /* must stay! */                 \
    register void *inKey__ = (searchKey__);                          \
    register void *userData__ = (self__)->userData;                  \
    register REDAOrderedDataTypeCompareFunction compareFnc__ =       \
            (self__)->nodeKeyCompareFnc;                             \
    register REDAOrderedDataTypeCompareWithUserDataFunction          \
            compareWithUserDataFnc =                                 \
                    (self__)->nodeKeyCompareWithUserDataFnc;         \
    register struct REDASkiplistNode *nextNode__ = NULL;             \
    register struct REDASkiplistNode *currNode__ = (self__)->_head;  \
    for (level__ = (int) ((self__)->_currentMaxLevel); level__ >= 0; \
            --level__) {                                             \
        while (((nextNode__ = REDASkiplistNode_getForwardPointer(    \
                        currNode__,                                  \
                        level__))                                    \
                != NULL)                                             \
                && (nextNode__->userData != NULL)                    \
                && ((compResult__ = compareWithUserDataFnc           \
                            ? compareWithUserDataFnc(                \
                                    nextNode__->userData,            \
                                    inKey__,                         \
                                    userData__)                      \
                            : REDAOrderedDataType_compareUsingFnc(   \
                                    compareFnc__,                    \
                                    nextNode__->userData,            \
                                    inKey__))                        \
                        < 0)) {                                      \
            currNode__ = nextNode__;                                 \
        }                                                            \
        /* invariant: compResult__>= 0 || nextNode__ == NULL */      \
        if (compResult__ == 0) {                                     \
            break;                                                   \
        }                                                            \
    }                                                                \
    if ((void *) preciseMatch__ != NULL) {                           \
        *((RTIBool *) preciseMatch__) = (compResult__ == 0);         \
    }                                                                \
    *(answer__) = nextNode__;                                        \
}


#ifdef __cplusplus
    }	/* extern "C" */
#endif

#endif /* reda_skiplist_impl_h */
