/**
 * (c) Copyright, Real-Time Innovations, 2013-2016.
 * All rights reserved.
 * No duplications, whole or partial, manual or electronic, may be made
 * without express written permission.  Any such copies, or
 * revisions thereof, must display this notice unaltered.
 * This code contains trade secrets of Real-Time Innovations, Inc.
 *
 * Description
 * -----------
 * Default create functions for security plugins
 * =====================================================================
 */
#ifndef security_access_control_h
#define security_access_control_h

#ifndef security_infrastructure_h
  #include "security/security_infrastructure.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

/* Property names according to DDS Security specification */
#define DDS_SECURITY_ACCESS_CONTROL_PROPERTY_PREFIX \
    DDS_TRUST_PLUGIN_PROPERTY_PREFIX "access."
#define DDS_SECURITY_PERMISSIONS_CA_PROPERTY \
    DDS_SECURITY_ACCESS_CONTROL_PROPERTY_PREFIX "permissions_ca"
#define DDS_SECURITY_GOVERNANCE_PROPERTY \
    DDS_SECURITY_ACCESS_CONTROL_PROPERTY_PREFIX "governance"
#define DDS_SECURITY_PERMISSIONS_PROPERTY \
    DDS_SECURITY_ACCESS_CONTROL_PROPERTY_PREFIX "permissions"
#define DDS_SECURITY_ACCESS_CONTROL_RTPS_PSK_PROTECTION_KIND_PROPERTY \
    DDS_SECURITY_ACCESS_CONTROL_PROPERTY_PREFIX "rtps_psk_protection_kind"

/* Property names used in 5.3.0 and below */
#define RTI_SECURITY_ACCESS_CONTROL_PROPERTY_PREFIX "access_control."
#define RTI_SECURITY_PERMISSIONS_AUTHORITY_FILE_PROPERTY \
    RTI_SECURITY_ACCESS_CONTROL_PROPERTY_PREFIX "permissions_authority_file"
#define RTI_SECURITY_PERMISSIONS_FILE_PROPERTY \
    RTI_SECURITY_ACCESS_CONTROL_PROPERTY_PREFIX "permissions_file"
#define RTI_SECURITY_GOVERNANCE_FILE_PROPERTY \
    RTI_SECURITY_ACCESS_CONTROL_PROPERTY_PREFIX "governance_file"

/* RTI-specific properties */
#define RTI_SECURITY_ALTERNATIVE_PERMISSIONS_AUTHORITY_FILES_PROPERTY \
    RTI_SECURITY_ACCESS_CONTROL_PROPERTY_PREFIX "alternative_permissions_authority_files"

/* Backward-compatibility properties */
#define RTI_SECURITY_ACCESS_CONTROL_530_LOGGING_PROTECTION_PROPERTY \
    RTI_SECURITY_ACCESS_CONTROL_PROPERTY_PREFIX "use_530_logging_protection"
#define RTI_SECURITY_ACCESS_CONTROL_530_PERMISSIONS_RULES_PRECEDENCE_PROPERTY \
    RTI_SECURITY_ACCESS_CONTROL_PROPERTY_PREFIX "use_530_permissions_rules_precedence"
#define RTI_SECURITY_ACCESS_CONTROL_530_PARTITIONS_PROPERTY \
    RTI_SECURITY_ACCESS_CONTROL_PROPERTY_PREFIX "use_530_partitions"
#define RTI_SECURITY_ACCESS_CONTROL_610_PERMISSIONS_RULES_PRECEDENCE_PROPERTY \
    RTI_SECURITY_ACCESS_CONTROL_PROPERTY_PREFIX "use_610_permissions_rules_precedence"

/* DDS Security Tables 60 and 62 */
#define RTI_SECURITY_PLUGIN_PARTICIPANT_SECURITY_ATTRIBUTES_FLAG_IS_RTPS_AXK_ENCRYPTED \
        (0x00000001U << 0U)
#define RTI_SECURITY_PLUGIN_PARTICIPANT_SECURITY_ATTRIBUTES_FLAG_IS_DISCOVERY_ENCRYPTED \
        (0x00000001U << 1U)
#define RTI_SECURITY_PLUGIN_PARTICIPANT_SECURITY_ATTRIBUTES_FLAG_IS_LIVELINESS_ENCRYPTED \
        (0x00000001U << 2U)

#define RTI_SECURITY_PLUGIN_PARTICIPANT_SECURITY_ATTRIBUTES_FLAG_IS_RTPS_ORIGIN_AUTHENTICATED \
        (0x00000001U << 3U)
#define RTI_SECURITY_PLUGIN_PARTICIPANT_SECURITY_ATTRIBUTES_FLAG_IS_DISCOVERY_ORIGIN_AUTHENTICATED \
        (0x00000001U << 4U)
#define RTI_SECURITY_PLUGIN_PARTICIPANT_SECURITY_ATTRIBUTES_FLAG_IS_LIVELINESS_ORIGIN_AUTHENTICATED \
        (0x00000001U << 5U)

#define RTI_SECURITY_PLUGIN_PARTICIPANT_SECURITY_ATTRIBUTES_FLAG_IS_RTPS_ENCRYPTED_WITH_PSK \
        (0x00000001U << 6U)

#define RTI_SECURITY_PLUGIN_PARTICIPANT_SECURITY_ATTRIBUTES_FLAG_IS_VALID \
        DDS_PLUGIN_PARTICIPANT_TRUST_ATTRIBUTES_FLAG_IS_VALID

#define RTI_SECURITY_PLUGIN_ENDPOINT_SECURITY_ATTRIBUTES_FLAG_IS_SUBMESSAGE_ENCRYPTED \
        (0x00000001U << 0U)
#define RTI_SECURITY_PLUGIN_ENDPOINT_SECURITY_ATTRIBUTES_FLAG_IS_PAYLOAD_ENCRYPTED \
        (0x00000001U << 1U)

#define RTI_SECURITY_PLUGIN_ENDPOINT_SECURITY_ATTRIBUTES_FLAG_IS_SUBMESSAGE_ORIGIN_AUTHENTICATED \
        (0x00000001U << 2U)

#define RTI_SECURITY_PLUGIN_ENDPOINT_SECURITY_ATTRIBUTES_FLAG_IS_VALID \
        DDS_PLUGIN_ENDPOINT_TRUST_ATTRIBUTES_FLAG_IS_VALID

/*
 * These flags identify the bits within the vendor participant protection info.
 * The vendor participant protection info bitmask is understood by both
 * core and the security plugins. These definitions must match those that are
 * defined in pres.1.0 (participant.ifc)
 */
#define RTI_SECURITY_VENDOR_PARTICIPANT_SECURITY_PROTECTION_INFO_FLAG_IS_SERVICE_REQUEST_PROTECTED \
        (0x00000001U << 0U)

#define RTI_SECURITY_VENDOR_PARTICIPANT_SECURITY_PROTECTION_INFO_FLAG_IS_SERVICE_REQUEST_ENCRYPTED \
        (0x00000001U << 1U)

#define RTI_SECURITY_VENDOR_PARTICIPANT_SECURITY_PROTECTION_INFO_FLAG_IS_SERVICE_REQUEST_ORIGIN_AUTHENTICATED \
        (0x00000001U << 2U)

#define RTI_SECURITY_VENDOR_PARTICIPANT_SECURITY_PROTECTION_INFO_FLAG_IS_INSTANCE_STATE_PROTECTED \
        (0x00000001U << 3U)

#define RTI_SECURITY_VENDOR_PARTICIPANT_SECURITY_PROTECTION_INFO_FLAG_IS_INSTANCE_STATE_ENCRYPTED \
        (0x00000001U << 4U)

#define RTI_SECURITY_VENDOR_PARTICIPANT_SECURITY_PROTECTION_INFO_FLAG_IS_INSTANCE_STATE_ORIGIN_AUTHENTICATED \
        (0x00000001U << 5U)

#define RTI_SECURITY_VENDOR_PARTICIPANT_SECURITY_PROTECTION_INFO_FLAG_IS_MONITORING_METRICS_PROTECTED \
        (0x00000001U << 6U)

#define RTI_SECURITY_VENDOR_PARTICIPANT_SECURITY_PROTECTION_INFO_FLAG_IS_MONITORING_METRICS_ENCRYPTED \
        (0x00000001U << 7U)

#define RTI_SECURITY_VENDOR_PARTICIPANT_SECURITY_PROTECTION_INFO_FLAG_IS_MONITORING_METRICS_ORIGIN_AUTHENTICATED \
        (0x00000001U << 8U)

#define RTI_SECURITY_VENDOR_PARTICIPANT_SECURITY_PROTECTION_INFO_FLAG_IS_MONITORING_LOGGING_PROTECTED \
        (0x00000001U << 9U)

#define RTI_SECURITY_VENDOR_PARTICIPANT_SECURITY_PROTECTION_INFO_FLAG_IS_MONITORING_LOGGING_ENCRYPTED \
        (0x00000001U << 10U)

#define RTI_SECURITY_VENDOR_PARTICIPANT_SECURITY_PROTECTION_INFO_FLAG_IS_MONITORING_LOGGING_ORIGIN_AUTHENTICATED \
        (0x00000001U << 11U)

#define RTI_SECURITY_VENDOR_PARTICIPANT_SECURITY_PROTECTION_INFO_FLAG_IS_VALID \
        (0x00000001U << 31U)

/*
 * Vendor Specific Endpoint Attributes
 * These bits are not used below dds_c.1.0, and are not serialized. They allow us
 * to provide information about vendor specific attributes without polluting the
 * plugin endpoint mask.
 */
#define RTI_SECURITY_VENDOR_ENDPOINT_SECURITY_ATTRIBUTES_FLAG_GROUP_BUILTIN_CRYPTO_DATA \
        DDS_VENDOR_ENDPOINT_TRUST_ATTRIBUTES_FLAG_GROUP_BUILTIN_INTERCEPTOR_DATA

#define DDS_SECURITY_ACCESS_CONTROL_PROTECTION_KIND_NONE_STRING "NONE"
#define DDS_SECURITY_ACCESS_CONTROL_PROTECTION_KIND_SIGN_STRING "SIGN"
#define DDS_SECURITY_ACCESS_CONTROL_PROTECTION_KIND_ENCRYPT_STRING "ENCRYPT"

#ifdef __cplusplus
}    /* extern "C" */
#endif

#endif /* security_access_control_h */