/**
 * (c) Copyright, Real-Time Innovations, 2022
 * All rights reserved.
 * No duplications, whole or partial, manual or electronic, may be made
 * without express written permission.  Any such copies, or
 * revisions thereof, must display this notice unaltered.
 * This code contains trade secrets of Real-Time Innovations, Inc.
 *
 * Description
 * -----------
 * RTI definitions of cryptography library structures and macros to enable
 * support for wolfSSL 4.7.0 (or lower, but note that we've only supported and
 * tested version 4.7.0).
 * A user application does not need to include this header file. It's only
 * intended for internal use by other connextdds modules.
 * =========================================================================
 */

#ifndef security_crypto_lib_adapter_wolfssl47_h
#define security_crypto_lib_adapter_wolfssl47_h

#if defined(RTI_WOLFSSL) \
    && (LIBWOLFSSL_VERSION_HEX <= RTI_WOLFSSL_VERSION_4_7_0)

/*
 * Needed because the values returned by X509_get_signature_nid differ between
 * OpenSSL and wolfSSL 4.7.0. wolfSSL 5.0.0 added the NID values to its
 * compatibility layer.
 */
#define RTI_SECURITY_SIGNATURE_ID_ECDSA_SHA384 CTC_SHA384wECDSA
#define RTI_SECURITY_SIGNATURE_ID_ECDSA_SHA256 CTC_SHA256wECDSA
#define RTI_SECURITY_SIGNATURE_ID_RSA_PKCS_SHA256 CTC_SHA256wRSA

int RTI_Security_CryptoLibAdapterWolfSSL_sk_X509_CRL_push(
        WOLF_STACK_OF_X509_CRL *stack,
        WOLFSSL_X509_CRL *crl);
#define sk_X509_CRL_push RTI_Security_CryptoLibAdapterWolfSSL_sk_X509_CRL_push

void RTI_Security_CryptoLibAdapterWolfSSL_sk_X509_CRL_pop_free(
        WOLF_STACK_OF_X509_CRL *stack,
        void(*f) (WOLFSSL_X509_CRL *));
#define sk_X509_CRL_pop_free RTI_Security_CryptoLibAdapterWolfSSL_sk_X509_CRL_pop_free

X509_CRL * RTI_Security_CryptoLibAdapterWolfSSL_sk_X509_CRL_value(
        WOLF_STACK_OF_X509_CRL *stack,
        int index);
#define sk_X509_CRL_value RTI_Security_CryptoLibAdapterWolfSSL_sk_X509_CRL_value

int RTI_Security_CryptoLibAdapterWolfSSL_sk_X509_CRL_num(
        WOLF_STACK_OF_X509_CRL *stack);
#define sk_X509_CRL_num RTI_Security_CryptoLibAdapterWolfSSL_sk_X509_CRL_num

/*
 * In wolfSSL 4.7.0, and according to
 * https://github.com/wolfSSL/wolfssl/blob/master/src/x509.c#L11069, some
 * flags were not supported. This issue affected the format of subject names
 * that the DDS_DomainParticipant_get_discovered_participant_subject_name API
 * returned. The format was slash-separated instead of comma separated in the
 * case of wolfSSL 4.7.0.
 */
#define RTI_SECURITY_SLASH_SEPARATED_SUBJECT_NAME

#endif /* RTI_WOLFSSL */
#endif /* security_crypto_lib_adapter_wolfssl47_h */