/*

 (c) Copyright, Real-Time Innovations, 2017.
 All rights reserved.

 No duplications, whole or partial, manual or electronic, may be made
 without express written permission.  Any such copies, or
 revisions thereof, must display this notice unaltered.
 This code contains trade secrets of Real-Time Innovations, Inc.

===================================================================== */
#ifndef security_infrastructure_psm_h
#define security_infrastructure_psm_h

/* This file is only used by Connext DDS Pro */
#include "ndds/ndds_c.h"

#include "log/log_makeheader.h"
#include "security/security_dll.h"

#include "dds_c/dds_c_trust_plugins.h"

#include "osapi/osapi_type.h"
#include "osapi/osapi_heap.h"

#ifdef __cplusplus
    extern "C" {
#endif

#define RTI_SECURITY_PLUGINS_SUPPORT_THREAD_SPECIFIC_STORAGE
#define RTI_SECURITY_PLUGINS_SUPPORT_FILE_TRACKER

/* Plugin class families for the security plugins */
#define RTI_SECURITY_AUTHENTICATION_CLASS_FAMILY \
  DDS_LOGGING_AUTHENTICATION_CLASS_FAMILY
#define RTI_SECURITY_ACCESS_CONTROL_CLASS_FAMILY \
  DDS_LOGGING_ACCESS_CONTROL_CLASS_FAMILY
#define RTI_SECURITY_CRYPTOGRAPHY_CLASS_FAMILY \
  DDS_LOGGING_INTERCEPTOR_CLASS_FAMILY
#define RTI_SECURITY_LOGGING_CLASS_FAMILY \
  DDS_LOGGING_LOGGING_CLASS_FAMILY

/* Plugin class names used for plugin logging and for defining class IDs */
#define RTI_SECURITY_COMMON_PLUGIN_CLASS "RTI:Common"
#define RTI_SECURITY_AUTHENTICATION_PLUGIN_CLASS \
  "DDS:" RTI_SECURITY_AUTHENTICATION_CLASS_FAMILY ":PKI-DH"
#define RTI_SECURITY_AUTHENTICATION_LIGHTWEIGHT_SECURITY_PLUGIN_CLASS \
  "DDS:" RTI_SECURITY_AUTHENTICATION_CLASS_FAMILY ":PSK"
#define RTI_SECURITY_ACCESS_CONTROL_PLUGIN_CLASS \
  "DDS:" RTI_SECURITY_ACCESS_CONTROL_CLASS_FAMILY ":Permissions"
#define RTI_SECURITY_ACCESS_CONTROL_LIGHTWEIGHT_SECURITY_PLUGIN_CLASS \
  "DDS:" RTI_SECURITY_ACCESS_CONTROL_CLASS_FAMILY ":PSK"
#define RTI_SECURITY_CRYPTOGRAPHY_PLUGIN_CLASS \
  "DDS:" RTI_SECURITY_CRYPTOGRAPHY_CLASS_FAMILY ":AES-GCM-GMAC"
#define RTI_SECURITY_CRYPTOGRAPHY_LIGHTWEIGHT_SECURITY_PLUGIN_CLASS \
  "DDS:" RTI_SECURITY_CRYPTOGRAPHY_CLASS_FAMILY ":PSK"
#define RTI_SECURITY_LOGGING_PLUGIN_CLASS \
  "DDS:" RTI_SECURITY_LOGGING_CLASS_FAMILY ":DDS_LogTopic"

/* Separator between class name and major version */
#define RTI_SECURITY_PLUGIN_CLASS_NAME_MAJOR_VERSION_SEPARATOR \
  PRES_PLUGIN_CLASS_NAME_MAJOR_VERSION_SEPARATOR

typedef struct REDAObjectPerWorker * RTI_Security_ThreadContextAccessorHandle;

typedef DDS_ProtectionKind RTI_Security_BasicProtectionKind;

#define RTI_SECURITY_NONE_PROTECTION_KIND DDS_NONE_PROTECTION_KIND
#define RTI_SECURITY_SIGN_PROTECTION_KIND DDS_SIGN_PROTECTION_KIND
#define RTI_SECURITY_ENCRYPT_PROTECTION_KIND DDS_TRANSFORM_PROTECTION_KIND
#define RTI_SECURITY_UNSET_PROTECTION_KIND DDS_UNSET_PROTECTION_KIND

struct DDS_CryptographyPlugin;
typedef struct DDS_CryptographyPlugin DDS_CryptographyPlugin;

typedef DDS_Interceptor_RegisterLocalParticipantFunction \
        RTI_Security_Cryptography_RegisterLocalParticipantFcn;

typedef DDS_Interceptor_RegisterMatchedRemoteParticipantFunction \
        RTI_Security_Cryptography_RegisterMatchedRemoteParticipantFcn;

typedef DDS_Interceptor_RegisterLocalEndpointFunction \
        RTI_Security_Cryptography_RegisterLocalEndpointFcn;

typedef DDS_Interceptor_RegisterMatchedRemoteEndpointFunction \
        RTI_Security_Cryptography_RegisterMatchedRemoteEndpointFcn;

typedef DDS_Interceptor_UnregisterFunction \
        RTI_Security_Cryptography_UnregisterFcn;

typedef DDS_Interceptor_CreateLocalInterceptorStateFunction \
        RTI_Security_Cryptography_CreateLocalCryptoTokensFcn;

typedef DDS_Interceptor_SetRemoteInterceptorStateFunction \
        RTI_Security_Cryptography_SetRemoteCryptoTokensFcn;

typedef DDS_Interceptor_SetLocalInterceptorStateFunction \
        RTI_Security_Cryptography_SetLocalCryptoTokensFcn;

typedef DDS_Interceptor_ReturnInterceptorStateFunction \
        RTI_Security_Cryptography_ReturnCryptoTokensFcn;

typedef DDS_Interceptor_TransformInterceptorStateFunction \
        RTI_Security_Cryptography_TransformLocalCryptoTokensFcn;

typedef DDS_Interceptor_TransformOutgoingSerializedDataFunction \
        RTI_Security_Cryptography_EncodeSerializedDataFcn;

typedef DDS_Interceptor_TransformOutgoingMetaDataFunction \
        RTI_Security_Cryptography_EncodeMetaDataFcn;

typedef DDS_Interceptor_TransformOutgoingRtpsMessageFunction \
        RTI_Security_Cryptography_EncodeRtpsMessageDataFcn;

typedef DDS_Interceptor_PreprocessInterceptorSubmsgFunction \
        RTI_Security_Cryptography_PreprocessSecureSubmsgFcn;

typedef DDS_Interceptor_TransformIncomingMetaDataFunction \
        RTI_Security_Cryptography_DecodeMetaDataFcn;

typedef DDS_Interceptor_TransformIncomingSerializedDataFunction \
        RTI_Security_Cryptography_DecodeSerializedDataFcn;

typedef DDS_Interceptor_GetTransformedOutgoingContentMaxSerializedSizeFunction \
        RTI_Security_Cryptography_GetEncodedContentMaxSerializedSizeFcn;

typedef DDS_Interceptor_CreateLocalKeyRevisionFunction \
        RTI_Security_Cryptography_CreateLocalKeyRevisionFcn;

typedef DDS_Interceptor_CreateLocalKeyRevisionTokensFunction \
        RTI_Security_Cryptography_CreateLocalKeyRevisionTokensFcn;

typedef DDS_Interceptor_SetRemoteKeyRevisionTokensFunction \
        RTI_Security_Cryptography_SetRemoteKeyRevisionTokensFcn;

typedef DDS_Interceptor_ActivateLocalKeyRevisionFunction \
        RTI_Security_Cryptography_ActivateLocalKeyRevisionFcn;

typedef DDS_Interceptor_RetransformOutgoingSerializedDataFunction \
        RTI_Security_Cryptography_ReencodeSerializedDataFcn;

typedef DDS_Interceptor_RetransformOutgoingSerializedDataFromDurableWriterHistoryFunction \
        RTI_Security_Cryptography_ReencodeSerializedDataFromDurableWriterHistoryFcn;

typedef DDS_Interceptor_GetEndpointSecAttributesFunction \
        RTI_Security_Cryptography_GetEndpointSecAttributesFcn;

typedef DDS_Interceptor_AssertPropertyQoSFunction \
        RTI_Security_Cryptography_AssertPropertyQoSFcn;

struct DDS_CryptographyPlugin {
    /* data that only the security plugin understands */
    void *plugin_data;
    struct DDS_LoggingPlugin *logging;

    /* CryptoKeyFactory interface */
    RTI_Security_Cryptography_RegisterLocalParticipantFcn
            register_local_participant;
    RTI_Security_Cryptography_RegisterMatchedRemoteParticipantFcn
            register_matched_remote_participant;
    RTI_Security_Cryptography_RegisterLocalEndpointFcn
            register_local_datawriter;
    RTI_Security_Cryptography_RegisterMatchedRemoteEndpointFcn
            register_matched_remote_datareader;
    RTI_Security_Cryptography_RegisterLocalEndpointFcn
            register_local_datareader;
    RTI_Security_Cryptography_RegisterMatchedRemoteEndpointFcn
            register_matched_remote_datawriter;

    RTI_Security_Cryptography_UnregisterFcn preunregister_datawriter;
    RTI_Security_Cryptography_UnregisterFcn preunregister_datareader;

    RTI_Security_Cryptography_UnregisterFcn unregister_participant;
    RTI_Security_Cryptography_UnregisterFcn unregister_datawriter;
    RTI_Security_Cryptography_UnregisterFcn unregister_datareader;

    /* CryptoKeyExchange interface */
    RTI_Security_Cryptography_CreateLocalCryptoTokensFcn
            create_local_participant_crypto_tokens;
    RTI_Security_Cryptography_SetRemoteCryptoTokensFcn
            set_remote_participant_crypto_tokens;
    RTI_Security_Cryptography_CreateLocalCryptoTokensFcn
            create_local_datawriter_crypto_tokens;
    RTI_Security_Cryptography_SetRemoteCryptoTokensFcn
            set_remote_datawriter_crypto_tokens;
    RTI_Security_Cryptography_CreateLocalCryptoTokensFcn
            create_local_datareader_crypto_tokens;
    RTI_Security_Cryptography_SetRemoteCryptoTokensFcn
            set_remote_datareader_crypto_tokens;
    RTI_Security_Cryptography_SetLocalCryptoTokensFcn
            set_local_datawriter_crypto_tokens;
    RTI_Security_Cryptography_ReturnCryptoTokensFcn
            return_crypto_tokens;

    /* CryptoTransform interface */
    RTI_Security_Cryptography_TransformLocalCryptoTokensFcn
            encode_local_datawriter_crypto_tokens;
    RTI_Security_Cryptography_TransformLocalCryptoTokensFcn
            decode_local_datawriter_crypto_tokens;
    RTI_Security_Cryptography_EncodeSerializedDataFcn encode_serialized_data;
    RTI_Security_Cryptography_EncodeMetaDataFcn encode_datawriter_submessage;
    RTI_Security_Cryptography_EncodeMetaDataFcn encode_datareader_submessage;
    RTI_Security_Cryptography_EncodeRtpsMessageDataFcn encode_rtps_message;
    RTI_Security_Cryptography_EncodeRtpsMessageDataFcn
            encode_rtps_message_with_preshared_secret;
    RTI_Security_Cryptography_PreprocessSecureSubmsgFcn preprocess_secure_submsg;
    RTI_Security_Cryptography_DecodeMetaDataFcn
            decode_rtps_message_with_preshared_secret;
    RTI_Security_Cryptography_DecodeMetaDataFcn decode_rtps_message;
    RTI_Security_Cryptography_DecodeMetaDataFcn decode_datawriter_submessage;
    RTI_Security_Cryptography_DecodeMetaDataFcn decode_datareader_submessage;
    RTI_Security_Cryptography_DecodeSerializedDataFcn decode_serialized_data;
    RTI_Security_Cryptography_GetEncodedContentMaxSerializedSizeFcn
            get_encoded_serialized_sample_size;
    RTI_Security_Cryptography_GetEncodedContentMaxSerializedSizeFcn
            get_encoded_datawriter_submessage_max_serialized_size;
    RTI_Security_Cryptography_GetEncodedContentMaxSerializedSizeFcn
            get_encoded_datareader_submessage_max_serialized_size;
    RTI_Security_Cryptography_GetEncodedContentMaxSerializedSizeFcn
            get_encoded_rtps_message_max_serialized_size;
    RTI_Security_Cryptography_GetEncodedContentMaxSerializedSizeFcn
            get_encoded_rtps_message_with_preshared_secret_max_serialized_size;
    /* key material regeneration and redistribution interface */
    RTI_Security_Cryptography_CreateLocalKeyRevisionFcn
            create_local_key_revision;
    RTI_Security_Cryptography_CreateLocalKeyRevisionTokensFcn
            create_local_key_revision_tokens;
    RTI_Security_Cryptography_SetRemoteKeyRevisionTokensFcn
            set_remote_key_revision_tokens;
    RTI_Security_Cryptography_ReturnCryptoTokensFcn
            return_local_key_revision_tokens;
    RTI_Security_Cryptography_ActivateLocalKeyRevisionFcn
            activate_local_key_revision;
    RTI_Security_Cryptography_ReencodeSerializedDataFcn
            reencode_serialized_data;
    RTI_Security_Cryptography_ReencodeSerializedDataFromDurableWriterHistoryFcn
            reencode_serialized_data_from_durable_writer_history;
    RTI_Security_Cryptography_AssertPropertyQoSFcn assert_property_qos;
};

#ifdef __cplusplus
    }   /* extern "C" */
#endif

#endif /* security_infrastructure_psm_h */