/*

 (c) Copyright, Real-Time Innovations, 2011-2016.
 All rights reserved.
 No duplications, whole or partial, manual or electronic, may be made
 without express written permission.  Any such copies, or
 revisions thereof, must display this notice unaltered.
 This code contains trade secrets of Real-Time Innovations, Inc.

=========================================================================*/
#ifndef security_log_h
#define security_log_h

#ifndef security_infrastructure_h
  #include "security/security_infrastructure.h"
#endif

#ifndef log_common_h
  #include "log/log_common.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

/*i \file
 * @brief Logging facilities for the security module
 */

/*e \mainpage
 *  RTI Data Distribution Service Security Library
 *
 */

/*e \ingroup RTI_SecurityLogModule
 * Submodule ID for plugin submodule
 */
#define SUBMODULE_SECURITY_COMMON         (0x1000)
#define SUBMODULE_SECURITY_AUTHENTICATION (0x2000)
#define SUBMODULE_SECURITY_ACCESS_CONTROL (0x4000)
#define SUBMODULE_SECURITY_CRYPTOGRAPHY   (0x8000)
#define SUBMODULE_SECURITY_LOGGING        (0x10000)

/*e \ingroup RTI_SecurityLogModule
 * Mask for turning on logging for only the plugin submodule
 */
#define RTI_SECURITY_SUBMODULE_MASK_COMMON                 (0x0001)
#define RTI_SECURITY_SUBMODULE_MASK_AUTHENTICATION         (0x0002)
#define RTI_SECURITY_SUBMODULE_MASK_ACCESS_CONTROL         (0x0004)
#define RTI_SECURITY_SUBMODULE_MASK_CRYPTOGRAPHY           (0x0008)
#define RTI_SECURITY_SUBMODULE_MASK_SECURITY               (0x0010)
#define RTI_SECURITY_SUBMODULE_MASK_LOGGING                (0x0020)
#define RTI_SECURITY_SUBMODULE_MASK_CRYPTO_LIB_ADAPTER     (0x0040)

/*e \ingroup RTI_SecurityLogModule
 * Mask for turning on logging for all submodules
 */
#define RTI_SECURITY_SUBMODULE_MASK_ALL              RTI_LOG_SUBMODULE_MASK_ALL

/*e \ingroup RTI_SecurityLogModule
 * Set a new bitmask value for the instrumentation bitmasks
 * of this module and those below it, using a convenient single verbosity value.
 *
 * This method also sets the same bitmask for the SECURITY log category.
 */
RTISecurityDllExport void
RTI_Security_Log_setGlobalDependencyInstrumentationMask(RTILogBitmap verbosity);

#ifdef __cplusplus
}    /* extern "C" */
#endif

#endif /* security_log_h */