/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)transport_udp.h    generated by: makeheader    Sat Mar  9 09:38:06 2024
 *
 *		built from:	udp.ifc
 */

#ifndef transport_udp_h
#define transport_udp_h


  #ifndef transport_dll_h
    #include "transport/transport_dll.h"
  #endif
  #ifndef transport_interface_h
    #include "transport/transport_interface.h"
  #endif
  #include "osapi/osapi_socket.h"
  #include "transport/transport_udp_wan_library.h"
#ifdef __cplusplus
    extern "C" {
#endif

struct RTIClock;

struct NDDS_Transport_UDP; /* forward declare */

struct NDDS_Transport_UDP_SocketFactory;

struct NDDS_Transport_UDP_Interface_t;


typedef void (*NDDS_Transport_UDP_WAN_Listener_Public_Mapping_Resolved_Callback)(
        void *listener_data,
        const NDDS_Transport_Address_t *public_uuid_adress,
        NDDS_Transport_Port_t rtps_port);


struct NDDS_Transport_UDP_WAN_Listener {
    void *listener_data;
    NDDS_Transport_UDP_WAN_Listener_Public_Mapping_Resolved_Callback on_public_mapping_resolved;
};

typedef RTI_UINT16 NDDS_Transport_UDP_Port;


#define NDDS_TRANSPORT_UDP_PORT_INVALID (0)
#define NDDS_TRANSPORT_UDP_PORT_MAX (RTI_UINT16_MAX)


struct NDDS_Transport_UDP_WAN_CommPortsMappingInfo {
    /*e \dref_UDPTransport_WAN_CommPortsMappingInfo_rtps_port
     */
    NDDS_Transport_Port_t rtps_port;
    /*e \dref_UDPTransport_WAN_CommPortsMappingInfo_host_port
     */
    NDDS_Transport_UDP_Port host_port;
    /*e \dref_UDPTransport_WAN_CommPortsMappingInfo_public_port
     */
    NDDS_Transport_UDP_Port public_port;
};

struct NDDS_Transport_UDP_Property_t {
    struct NDDS_Transport_Property_t parent;
    RTI_INT32 send_socket_buffer_size;
    RTI_INT32 recv_socket_buffer_size;
    RTI_INT32 unicast_enabled;
    RTI_INT32 multicast_enabled;
    RTI_INT32 multicast_ttl;
    RTI_INT32 multicast_loopback_disabled;
    RTI_INT32 ignore_loopback_interface;
    RTI_INT32 ignore_nonup_interfaces;
    RTI_INT32 ignore_nonrunning_interfaces;
    RTI_INT32 no_zero_copy;
    RTI_INT32 send_blocking;
    RTI_INT32 enable_v4mapped;
    RTI_INT32 use_checksum;
    RTI_UINT32 transport_priority_mask;
    RTI_INT32 transport_priority_mapping_low;
    RTI_INT32 transport_priority_mapping_high;
    RTI_INT32 send_ping;
    RTI_INT32 force_interface_poll_detection;
    RTI_UINT32 interface_poll_period;
    RTI_INT32 reuse_multicast_receive_resource;
    RTI_INT32 protocol_overhead_max;
    RTI_INT32 disable_interface_tracking;
    RTI_UINT32 join_multicast_group_timeout;
    char * public_address;
    struct NDDS_Transport_UDP_WAN_CommPortsMappingInfo *comm_ports_list;
    RTI_INT32 comm_ports_list_length;
    struct NDDS_Transport_UDP_WAN_Listener *wan_listener;
    RTI_INT32 port_offset;
    RTI_UINT32 binding_ping_period;
    /*i
     * \brief If this property is set, we use this function to check for an
     * enabled WAN transport. If the function is not set, we would try to load a
     * default shared library with the correct function to enable WAN transport.
     */
    NDDS_Transport_UDP_WAN_Library_Is_Plugin_Enabled_Function
            plugin_enabled_function_ptr;
    /*i
     * \brief If this property is set we will update the send
     * and receive statistics split into participant and endpoint discovery
     * by parsing the packets. We will update the discovery/user packets
     * based on the port number regardless of if this property is set.
     * The default is false for performance reasons and is set to true only 
     * for testing purposes.
     */
    RTI_INT32 gather_detailed_statistics;
};


#define NDDS_TRANSPORT_UDP_PROPERTIES_BITMAP_DEFAULT (0)


#define NDDS_TRANSPORT_UDP_GATHER_SEND_BUFFER_COUNT_MAX_DEFAULT (16)


#define NDDS_TRANSPORT_UDP_SOCKET_BUFFER_SIZE_OS_DEFAULT (-1)


#define NDDS_TRANSPORT_UDP_SEND_SOCKET_BUFFER_SIZE_DEFAULT (131072)


#define NDDS_TRANSPORT_UDP_RECV_SOCKET_BUFFER_SIZE_DEFAULT (131072)


#define NDDS_TRANSPORT_UDP_DATAGRAM_LENGTH_MAX (65535)


#define NDDS_TRANSPORT_UDP_MULTICAST_TTL_DEFAULT (1)


#ifdef RTI_MULTICAST
  #define NDDS_TRANSPORT_UDP_USE_MULTICAST_DEFAULT 1
#else
  #define NDDS_TRANSPORT_UDP_USE_MULTICAST_DEFAULT 0
#endif


/*e \dref_UDPTransport_IGNORE_LOOPBACK_DISABLED
 */
  #define NDDS_TRANSPORT_UDP_IGNORE_LOOPBACK_DISABLED        (0)

/*e \dref_UDPTransport_IGNORE_LOOPBACK_ENABLED
 */
  #define NDDS_TRANSPORT_UDP_IGNORE_LOOPBACK_ENABLED         (1)

/*e \dref_UDPTransport_IGNORE_LOOPBACK_AUTO
 */
  #define NDDS_TRANSPORT_UDP_IGNORE_LOOPBACK_AUTO            (-1)


/*e \dref_UDPTransport_BLOCKING_NEVER
 */
  #define NDDS_TRANSPORT_UDP_BLOCKING_NEVER        (0)

/*e \dref_UDPTransport_BLOCKING_ALWAYS
 */
  #define NDDS_TRANSPORT_UDP_BLOCKING_ALWAYS       (1)
  #define NDDS_TRANSPORT_UDP_BLOCKING_UNICAST_ONLY (2)

/*e \dref_UDPTransport_BLOCKING_DEFAULT
 */
  #define NDDS_TRANSPORT_UDP_BLOCKING_DEFAULT \
            NDDS_TRANSPORT_UDP_BLOCKING_ALWAYS

struct NDDS_Transport_UDP_InterfaceListener;/*forward declare */

typedef 
RTI_INT32 (*NDDS_Transport_UDP_Interface_Filter_Fcn)(
    struct NDDS_Transport_UDP_InterfaceListener* self,
    struct NDDS_Transport_UDP        *plugin,
    RTI_INT32                        *multicast_enabled_out,
    NDDS_Transport_Interface_t       *interface_in,
    const char*                       interface_name_in);

struct NDDS_Transport_UDP_InterfaceListener {
    /* \copydoc NDDS_Transport_UDP_Interface_Filter_Fcn
     */
    NDDS_Transport_UDP_Interface_Filter_Fcn onInterface;
};

struct NDDS_Transport_UDP_SocketFactory;

typedef RTI_INT32
(*NDDS_Transport_UDP_SocketFactory_Send_Socket_Create_Fcn)(
    struct NDDS_Transport_UDP_SocketFactory         *me,
    NDDS_Transport_Port_t                           *port_out,
    const struct NDDS_Transport_UDP                 *transport_in,
    const NDDS_Transport_Address_t                  *interface_address_in,
    const int		    	                    *interface_index_in);

typedef RTI_INT32 (*NDDS_Transport_UDP_SocketFactory_Receive_Socket_Create_Fcn)(
    struct NDDS_Transport_UDP_SocketFactory         *me,
    NDDS_Transport_Port_t                           *port_inout,
    const struct NDDS_Transport_UDP                 *transport_in, 
    const NDDS_Transport_Address_t                  *multicast_address_in);

typedef RTI_INT32 (*NDDS_Transport_UDP_SocketFactory_Socket_Destroy_Fcn)(
    struct NDDS_Transport_UDP_SocketFactory   *me, 
    RTI_INT32                                  s_in);

struct NDDS_Transport_UDP_SocketFactory {

    /*e Children: implement this method. */
    NDDS_Transport_UDP_SocketFactory_Send_Socket_Create_Fcn    create_send_socket;

    /*e Children: implement this method. */
    NDDS_Transport_UDP_SocketFactory_Receive_Socket_Create_Fcn create_receive_socket;

    /*e Children: implement this method to destroy both the send and
     receive sockets. */
    NDDS_Transport_UDP_SocketFactory_Socket_Destroy_Fcn        destroy_socket;
};


#define NDDS_Transport_UDP_SocketFactory_is_valid(m) ( \
       (m)->create_send_socket    && \
       (m)->create_receive_socket && \
       (m)->destroy_socket )

extern NDDS_Transport_DllExport 
const char * NDDS_Transport_UDP_get_class_name_cEA(
    NDDS_Transport_Plugin    *self);

extern NDDS_Transport_DllExport 
RTI_INT32 NDDS_Transport_UDP_string_to_address_cEA(
    NDDS_Transport_Plugin    *self,
    NDDS_Transport_Address_t *address_out,
    const char               *address_in);

extern NDDS_Transport_DllExport 
NDDS_Transport_Plugin *NDDS_Transport_UDP_new(
    const struct NDDS_Transport_UDP_Property_t *property_in);

extern NDDS_Transport_DllExport 
NDDS_Transport_Plugin *NDDS_Transport_UDP_newI(
    const struct NDDS_Transport_UDP_Property_t *property_in,
    struct NDDS_Transport_UDP_SocketFactory    *socket_factory_in,
    struct NDDS_Transport_UDP_InterfaceListener* interface_listener_in,
    struct RTIClock                              *clock_in);

extern NDDS_Transport_DllExport 
RTI_INT32 NDDS_Transport_UDP_get_receive_interfaces_cEA(
    NDDS_Transport_Plugin      *self,
    RTI_INT32                  *found_more_than_provided_for_out,
    RTI_INT32                  *interface_reported_count_out,
    NDDS_Transport_Interface_t  interface_array_inout[],
    RTI_INT32                   interface_array_size_in);

extern NDDS_Transport_DllExport 
RTI_INT32 NDDS_Transport_UDP_get_ignore_loopback_interface_status(
    const NDDS_Transport_Plugin * self);

extern NDDS_Transport_DllExport
RTIBool NDDS_Transport_UDP_is_local_address(
    const NDDS_Transport_Plugin * self,
    const NDDS_Transport_Address_t *address_in);

extern NDDS_Transport_DllExport
RTI_INT32 NDDS_Transport_UDP_map_transport_priority(
    struct NDDS_Transport_UDP_Property_t * property,
    RTI_INT32                       transport_priority_in);


#define NDDS_TRANSPORT_UDP_UNBLOCK_MESSAGE_SIZE \
    (4 + 4 + sizeof(NDDS_Transport_Plugin *))


#define NDDS_TRANSPORT_UDP_DIFF_SERV_MAX_VALUE RTI_OSAPI_SOCKET_MAX_DIFF_SERV


struct NDDS_Transport_UDP_Statistics {
    /***********************
     *   PUBLIC FIELDS
     ***********************/
    RTI_UINT64 user_bytes_sent;
    RTI_UINT64 user_bytes_received;
    RTI_UINT64 user_packets_sent;
    RTI_UINT64 user_packets_received;
    /* 
     * discovery_bytes and discovery_packets are calculated using the
     * port number that is sent to/received from. This assumes the port
     * is an even number. When using an odd port number (ie, using an odd
     * domain ID), these statistics will not be calculated correctly.
     * You should not use these members in tests where the domain ID
     * and port numbers may change.
     */
    RTI_UINT64 discovery_bytes_sent;
    RTI_UINT64 discovery_bytes_received;
    RTI_UINT64 discovery_packets_sent;
    RTI_UINT64 discovery_packets_received;
    /*
     * participant_discovery_bytes and participant_discovery_packets refer to
     * DATA(P)s or participant bootstrap messages (i.e. messages sent/received
     * by the anonymous participant writer/reader).
     */
    RTI_UINT64 participant_discovery_bytes_sent;
    RTI_UINT64 participant_discovery_bytes_received;
    RTI_UINT64 participant_discovery_packets_sent;
    RTI_UINT64 participant_discovery_packets_received;
    /* 
     * participant_config_bytes and participant_config_packets refer to 
     * participant configuration announcements sent reliably between 
     * participants.
     */
    RTI_UINT64 participant_config_bytes_sent;
    RTI_UINT64 participant_config_bytes_received;
    RTI_UINT64 participant_config_packets_sent;
    RTI_UINT64 participant_config_packets_received;
    RTI_UINT64 automatic_liveliness_bytes_sent;
    RTI_UINT64 automatic_liveliness_bytes_received;
    RTI_UINT64 automatic_liveliness_packets_sent;
    RTI_UINT64 automatic_liveliness_packets_received;
    RTI_UINT64 manual_liveliness_bytes_sent;
    RTI_UINT64 manual_liveliness_bytes_received;
    RTI_UINT64 manual_liveliness_packets_sent;
    RTI_UINT64 manual_liveliness_packets_received;
    RTI_UINT64 endpoint_discovery_bytes_sent;
    RTI_UINT64 endpoint_discovery_bytes_received;
    RTI_UINT64 endpoint_discovery_packets_sent;
    RTI_UINT64 endpoint_discovery_packets_received;
    /* 
     * Overhead statistics are for packets that use a builtin port (so are not
     * user data), but are also not associated with participant or endpoint
     * discovery
     */
    RTI_UINT64 overhead_bytes_sent;
    RTI_UINT64 overhead_bytes_received;
    RTI_UINT64 overhead_packets_sent;
    RTI_UINT64 overhead_packets_received;
    RTI_UINT64 binding_ping_event_count;
    RTI_UINT64 binding_ping_packets_sent;
    RTI_UINT64 binding_ping_bytes_sent;
    RTI_UINT64 binding_ping_packets_received;
    RTI_UINT64 binding_ping_bytes_received;

    /* 
     * Array of counters, one per DiffServ value. These counters will be
     * incremented based on the received DiffServ values of the UDP packet.
     * 
     * If received_priority_count[2] == 45 it means that we have received 45
     * packets with DiffServ value 2.
     * 
     * This field is populated only when gather_detailed_statistics is set
     * to 1.
     */
    RTI_UINT32
            received_priority_count[NDDS_TRANSPORT_UDP_DIFF_SERV_MAX_VALUE + 1];

    /***********************
     *   PRIVATE FIELDS
     ***********************/
    /* 
     * This field is used differently depending on whether or not we are using
     * security. If we are not using security, this field will count the
     * automatic liveliness messages sent and received. If we are using 
     * security, this field will count both automatic and manual liveliness
     * messages sent and received.
     */
    RTI_UINT64 _liveliness_bytes_sent;
    RTI_UINT64 _liveliness_bytes_received;
    RTI_UINT64 _liveliness_packets_sent;
    RTI_UINT64 _liveliness_packets_received;
};


#define NDDS_Transport_UDP_Statistics_INITIALIZER { \
    0, /* user_bytes_sent */ \
    0, /* user_bytes_received */ \
    0, /* user_packets_sent */ \
    0, /* user_packets_received */ \
    0, /* discovery_bytes_sent */ \
    0, /* discovery_bytes_received */ \
    0, /* discovery_packets_sent */ \
    0, /* discovery_packets_received */ \
    0, /* participant_discovery_bytes_sent */ \
    0, /* participant_discovery_bytes_received */ \
    0, /* participant_discovery_packets_sent */ \
    0, /* participant_discovery_packets_received */ \
    0, /* participant_config_bytes_sent */ \
    0, /* participant_config_bytes_received */ \
    0, /* participant_config_packets_sent */ \
    0, /* participant_config_packets_received */ \
    0, /* automatic_liveliness_bytes_sent */ \
    0, /* automatic_liveliness_bytes_received */ \
    0, /* automatic_liveliness_packets_sent */ \
    0, /* automatic_liveliness_packets_received */ \
    0, /* manual_liveliness_bytes_sent */ \
    0, /* manual_liveliness_bytes_received */ \
    0, /* manual_liveliness_packets_sent */ \
    0, /* manual_liveliness_packets_received */ \
    0, /* endpoint_discovery_bytes_sent */ \
    0, /* endpoint_discovery_bytes_received */ \
    0, /* endpoint_discovery_packets_sent */ \
    0, /* endpoint_discovery_packets_received */ \
    0, /* overhead_bytes_sent */ \
    0, /* overhead_bytes_received */ \
    0, /* overhead_packets_sent */ \
    0, /* overhead_packets_received */ \
    0, /* binding_ping_event_count */ \
    0, /* binding_ping_packets_sent */ \
    0, /* binding_ping_bytes_sent */ \
    0, /* binding_ping_packets_received */ \
    0, /* binding_ping_bytes_received */ \
    {0}, /* received_priority */ \
    0, /* _liveliness_bytes_sent */ \
    0, /* _liveliness_bytes_received */ \
    0, /* _liveliness_packets_sent */ \
    0 /* _liveliness_packets_received */ \
}

extern NDDS_Transport_DllExport
void NDDS_Transport_UDP_get_statistics(
    NDDS_Transport_Plugin    *self,
    struct NDDS_Transport_UDP_Statistics * statistics_out);

extern NDDS_Transport_DllExport
void NDDS_Transport_UDP_get_priority_range(
    NDDS_Transport_Plugin *self,
    RTI_INT32 *min_priority, 
    RTI_INT32 *max_priority);

extern NDDS_Transport_DllExport
void NDDS_Transport_UDP_Address_print(
    const NDDS_Transport_Address_t *me, 
    const char *desc,
    RTI_INT32 indent);


typedef struct {
    RTI_UINT8 value[9];
} NDDS_Transport_UDP_WAN_UUID;

extern NDDS_Transport_DllVariable
const NDDS_Transport_UDP_WAN_UUID NDDS_Transport_UDP_WAN_UUID_ZERO;

extern NDDS_Transport_DllExport
RTIBool NDDS_Transport_UDP_WAN_UUID_is_zero(NDDS_Transport_UDP_WAN_UUID *uuid);

extern NDDS_Transport_DllExport
RTIBool NDDS_Transport_UDP_WAN_UUID_populate(
        NDDS_Transport_UDP_WAN_UUID *self,
        const struct RTIOsapiRtpsGuidPrefix *guidPrefix,
        RTI_UINT32 hostAddress);


typedef RTI_UINT8 NDDS_Transport_UDP_WAN_AddressFlag;



#define NDDS_TRANSPORT_UDP_WAN_ADDRESS_FLAG_NONE (0)

/*
 * The U flag indicates if the locator contains a UUID. While this identifier by
 * itself cannot be directly used to reach the transport instance, the UUID can
 * be mapped to a public IP  address by  other Connext DDS applications and
 * services. Also, a locator can have both, the U flag, and the P flag, enabled
 * simultaneously. Locators with the U flag set are called UUID locators.
 *
 */
#define NDDS_TRANSPORT_UDP_WAN_ADDRESS_FLAG_U (0x01 << 0)
/*
 * The P flag indicates that the locator contains a globally public IP address
 * and public port where a transport instance can be reached. public_ip_address
 * contains the public IP address and public_port contains the public UDP port.
 * The port is always used to receive data and if the B flag is set it is also
 * used to send data. Locators with the P flag set are called PUBLIC locators.
 *
 */
#define NDDS_TRANSPORT_UDP_WAN_ADDRESS_FLAG_P (0x01 << 1)
/*
 * The B flag indicates if the locator is unidirectional or bidirectional.
 * Bidirectional locators can send/receive RTPS traffic. Unidirectional locators
 * can only receive RTPS traffic.
 */
#define NDDS_TRANSPORT_UDP_WAN_ADDRESS_FLAG_B (0x01 << 2)
/*
 * The R flag indicates if the locator is a relay locator provided by a Relay
 * Service. Locators with the R flag set are called RELAY locators.
 *
 */
#define NDDS_TRANSPORT_UDP_WAN_ADDRESS_FLAG_R (0x01 << 3)


typedef struct {
    RTI_UINT8 network_ordered_value[2];
} NDDS_Transport_UDP_WAN_NetworkPort;


#define NDDS_TRANSPORT_UDP_WAN_NETWORK_PORT_INVALID {{0}}


typedef struct {
    RTI_UINT8 network_ordered_value[4];
} NDDS_Transport_UDP_WAN_NetworkIpv4Address;


#define NDDS_TRANSPORT_UDP_WAN_NETWORK_IP_ADDRESS_INVALID {{0}}


typedef struct {
    NDDS_Transport_UDP_WAN_AddressFlag _flags;
    NDDS_Transport_UDP_WAN_UUID _uuid;
    NDDS_Transport_UDP_WAN_NetworkPort _publicPort;
    NDDS_Transport_UDP_WAN_NetworkIpv4Address _publicAddress;
} NDDS_Transport_UDPv4_WAN_Address;

extern NDDS_Transport_DllExport
NDDS_Transport_UDP_Port NDDS_Transport_UDPv4_WAN_Address_get_public_port(
        const NDDS_Transport_UDPv4_WAN_Address *self);

extern NDDS_Transport_DllExport
NDDS_Transport_UDP_Port NDDS_Transport_UDPv4_WAN_Address_get_public_port_for_rtps_port(
        const NDDS_Transport_UDPv4_WAN_Address *self,
        NDDS_Transport_Port_t rtps_port,
        RTI_INT32 udp_port_offset);

extern NDDS_Transport_DllExport
void NDDS_Transport_UDPv4_WAN_Address_set_public_port(
        NDDS_Transport_UDPv4_WAN_Address *self,
        NDDS_Transport_UDP_Port public_port);

extern NDDS_Transport_DllExport
RTI_UINT32 NDDS_Transport_UDPv4_WAN_Address_get_public_address(
        const NDDS_Transport_UDPv4_WAN_Address *self);

extern NDDS_Transport_DllExport
void NDDS_Transport_UDPv4_WAN_Address_set_public_address(
        NDDS_Transport_UDPv4_WAN_Address *self,
        RTI_UINT32 public_address);

extern NDDS_Transport_DllExport
void NDDS_Transport_UDP_combine_addresses(
        const NDDS_Transport_Plugin *self,
        NDDS_Transport_Address_t *address_out,
        const NDDS_Transport_Address_t *first_address_in,
        const NDDS_Transport_Address_t *second_address_in);


#ifdef __cplusplus
    }	/* extern "C" */
#endif

  #include "transport/transport_udp_impl.h"

#endif /* transport_udp_h */
